import sys
from typing import Any, IO, Optional, Tuple, Callable, Dict, List, Union, Protocol, Type

from .decoder import JSONDecoder as JSONDecoder
from .encoder import JSONEncoder as JSONEncoder
from .decoder import JSONDecodeError as JSONDecodeError

def dumps(obj: Any,
          skipkeys: bool = ...,
          ensure_ascii: bool = ...,
          check_circular: bool = ...,
          allow_nan: bool = ...,
          cls: Optional[Type[JSONEncoder]] = ...,
          indent: Union[None, int, str] = ...,
          separators: Optional[Tuple[str, str]] = ...,
          default: Optional[Callable[[Any], Any]] = ...,
          sort_keys: bool = ...,
          **kwds: Any) -> str: ...

def dump(obj: Any,
         fp: IO[str],
         skipkeys: bool = ...,
         ensure_ascii: bool = ...,
         check_circular: bool = ...,
         allow_nan: bool = ...,
         cls: Optional[Type[JSONEncoder]] = ...,
         indent: Union[None, int, str] = ...,
         separators: Optional[Tuple[str, str]] = ...,
         default: Optional[Callable[[Any], Any]] = ...,
         sort_keys: bool = ...,
         **kwds: Any) -> None: ...

if sys.version_info >= (3, 6):
    _LoadsString = Union[str, bytes, bytearray]
else:
    _LoadsString = str
def loads(s: _LoadsString,
          encoding: Any = ...,  # ignored and deprecated
          cls: Optional[Type[JSONDecoder]] = ...,
          object_hook: Optional[Callable[[Dict[Any, Any]], Any]] = ...,
          parse_float: Optional[Callable[[str], Any]] = ...,
          parse_int: Optional[Callable[[str], Any]] = ...,
          parse_constant: Optional[Callable[[str], Any]] = ...,
          object_pairs_hook: Optional[Callable[[List[Tuple[Any, Any]]], Any]] = ...,
          **kwds: Any) -> Any: ...

class _Reader(Protocol):
    def read(self) -> _LoadsString: ...

def load(fp: _Reader,
         cls: Optional[Type[JSONDecoder]] = ...,
         object_hook: Optional[Callable[[Dict[Any, Any]], Any]] = ...,
         parse_float: Optional[Callable[[str], Any]] = ...,
         parse_int: Optional[Callable[[str], Any]] = ...,
         parse_constant: Optional[Callable[[str], Any]] = ...,
         object_pairs_hook: Optional[Callable[[List[Tuple[Any, Any]]], Any]] = ...,
         **kwds: Any) -> Any: ...



from datetime import datetime
from typing import Any, Iterable, List, Optional, Tuple, Type, Union, Mapping, TypeVar, Generic

from django.db.backends.sqlite3.base import DatabaseWrapper
from django.db.models.expressions import Expression, Func
from django.db.models.query_utils import RegisterLookupMixin
from django.db.models.sql.compiler import SQLCompiler
from django.utils.datastructures import OrderedSet
from django.utils.safestring import SafeText

from django.db.models.fields import TextField, related_lookups

_T = TypeVar("_T")

class Lookup(Generic[_T]):
    lookup_name: str = ...
    prepare_rhs: bool = ...
    can_use_none_as_rhs: bool = ...
    lhs: Any = ...
    rhs: Any = ...
    bilateral_transforms: List[Type[Transform]] = ...
    def __init__(self, lhs: Union[Expression, TextField, related_lookups.MultiColSource], rhs: Any) -> None: ...
    def apply_bilateral_transforms(self, value: Expression) -> Transform: ...
    def batch_process_rhs(
        self, compiler: SQLCompiler, connection: DatabaseWrapper, rhs: Optional[OrderedSet] = ...
    ) -> Tuple[List[str], List[str]]: ...
    def get_source_expressions(self) -> List[Expression]: ...
    def set_source_expressions(self, new_exprs: List[Expression]) -> None: ...
    def get_prep_lookup(self) -> Any: ...
    def get_db_prep_lookup(self, value: Union[int, str], connection: DatabaseWrapper) -> Tuple[str, List[SafeText]]: ...
    def process_lhs(
        self, compiler: SQLCompiler, connection: DatabaseWrapper, lhs: Optional[Expression] = ...
    ) -> Tuple[str, List[Union[int, str]]]: ...
    def process_rhs(self, compiler: SQLCompiler, connection: DatabaseWrapper) -> Tuple[str, List[Union[int, str]]]: ...
    def rhs_is_direct_value(self) -> bool: ...
    def relabeled_clone(self: _T, relabels: Mapping[str, str]) -> _T: ...
    def get_group_by_cols(self) -> List[Expression]: ...
    def as_sql(self, compiler: Any, connection: Any) -> Any: ...
    def contains_aggregate(self) -> bool: ...
    def contains_over_clause(self) -> bool: ...
    @property
    def is_summary(self) -> bool: ...

class Transform(RegisterLookupMixin, Func):
    bilateral: bool = ...
    @property
    def lhs(self) -> Expression: ...
    def get_bilateral_transforms(self) -> List[Type[Transform]]: ...

class BuiltinLookup(Lookup[_T]):
    def get_rhs_op(self, connection: DatabaseWrapper, rhs: str) -> str: ...

class FieldGetDbPrepValueMixin:
    get_db_prep_lookup_value_is_iterable: bool = ...

class FieldGetDbPrepValueIterableMixin(FieldGetDbPrepValueMixin):
    def get_prep_lookup(self) -> Iterable[Any]: ...
    def resolve_expression_parameter(
        self, compiler: SQLCompiler, connection: DatabaseWrapper, sql: str, param: Any
    ) -> Any: ...

class Exact(FieldGetDbPrepValueMixin, BuiltinLookup): ...
class IExact(BuiltinLookup): ...
class GreaterThan(FieldGetDbPrepValueMixin, BuiltinLookup): ...
class GreaterThanOrEqual(FieldGetDbPrepValueMixin, BuiltinLookup[_T]): ...
class LessThan(FieldGetDbPrepValueMixin, BuiltinLookup[_T]): ...
class LessThanOrEqual(FieldGetDbPrepValueMixin, BuiltinLookup): ...

class IntegerFieldFloatRounding:
    rhs: Any = ...
    def get_prep_lookup(self) -> Any: ...

class IntegerGreaterThanOrEqual(IntegerFieldFloatRounding, GreaterThanOrEqual[Union[int, float]]): ...
class IntegerLessThan(IntegerFieldFloatRounding, LessThan[Union[int, float]]): ...

class In(FieldGetDbPrepValueIterableMixin, BuiltinLookup):
    def split_parameter_list_as_sql(self, compiler: Any, connection: Any): ...

class PatternLookup(BuiltinLookup[str]):
    param_pattern: str = ...

class Contains(PatternLookup): ...
class IContains(Contains): ...
class StartsWith(PatternLookup): ...
class IStartsWith(StartsWith): ...
class EndsWith(PatternLookup): ...
class IEndsWith(EndsWith): ...
class Range(FieldGetDbPrepValueIterableMixin, BuiltinLookup): ...
class IsNull(BuiltinLookup[bool]): ...
class Regex(BuiltinLookup[str]): ...
class IRegex(Regex): ...

class YearLookup(Lookup):
    def year_lookup_bounds(self, connection: DatabaseWrapper, year: int) -> List[str]: ...

class YearComparisonLookup(YearLookup):
    def get_rhs_op(self, connection: DatabaseWrapper, rhs: str) -> str: ...
    def get_bound(self, start: datetime, finish: datetime) -> Any: ...

class YearExact(YearLookup, Exact): ...
class YearGt(YearComparisonLookup): ...
class YearGte(YearComparisonLookup): ...
class YearLt(YearComparisonLookup): ...
class YearLte(YearComparisonLookup): ...



from typing import Any, Callable, Dict, Optional, Text, Union, overload

class Key:
    DefaultContentType: str
    RestoreBody: str
    BufferSize: Any
    base_user_settable_fields: Any
    base_fields: Any
    bucket: Any
    name: str
    metadata: Any
    cache_control: Any
    content_type: Any
    content_encoding: Any
    content_disposition: Any
    content_language: Any
    filename: Any
    etag: Any
    is_latest: bool
    last_modified: Any
    owner: Any
    path: Any
    resp: Any
    mode: Any
    size: Any
    version_id: Any
    source_version_id: Any
    delete_marker: bool
    encrypted: Any
    ongoing_restore: Any
    expiry_date: Any
    local_hashes: Any
    def __init__(self, bucket: Optional[Any] = ..., name: Optional[Any] = ...) -> None: ...
    def __iter__(self): ...
    @property
    def provider(self): ...
    key: Any
    md5: Any
    base64md5: Any
    storage_class: Any
    def get_md5_from_hexdigest(self, md5_hexdigest): ...
    def handle_encryption_headers(self, resp): ...
    def handle_version_headers(self, resp, force: bool = ...): ...
    def handle_restore_headers(self, response): ...
    def handle_addl_headers(self, headers): ...
    def open_read(
        self,
        headers: Optional[Dict[Text, Text]] = ...,
        query_args: str = ...,
        override_num_retries: Optional[Any] = ...,
        response_headers: Optional[Dict[Text, Text]] = ...,
    ): ...
    def open_write(self, headers: Optional[Dict[Text, Text]] = ..., override_num_retries: Optional[Any] = ...): ...
    def open(
        self,
        mode: str = ...,
        headers: Optional[Dict[Text, Text]] = ...,
        query_args: Optional[Any] = ...,
        override_num_retries: Optional[Any] = ...,
    ): ...
    closed: bool
    def close(self, fast: bool = ...): ...
    def next(self): ...
    __next__: Any
    def read(self, size: int = ...): ...
    def change_storage_class(self, new_storage_class, dst_bucket: Optional[Any] = ..., validate_dst_bucket: bool = ...): ...
    def copy(
        self,
        dst_bucket,
        dst_key,
        metadata: Optional[Any] = ...,
        reduced_redundancy: bool = ...,
        preserve_acl: bool = ...,
        encrypt_key: bool = ...,
        validate_dst_bucket: bool = ...,
    ): ...
    def startElement(self, name, attrs, connection): ...
    def endElement(self, name, value, connection): ...
    def exists(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def delete(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def get_metadata(self, name): ...
    def set_metadata(self, name, value): ...
    def update_metadata(self, d): ...
    def set_acl(self, acl_str, headers: Optional[Dict[Text, Text]] = ...): ...
    def get_acl(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def get_xml_acl(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def set_xml_acl(self, acl_str, headers: Optional[Dict[Text, Text]] = ...): ...
    def set_canned_acl(self, acl_str, headers: Optional[Dict[Text, Text]] = ...): ...
    def get_redirect(self): ...
    def set_redirect(self, redirect_location, headers: Optional[Dict[Text, Text]] = ...): ...
    def make_public(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def generate_url(
        self,
        expires_in,
        method: str = ...,
        headers: Optional[Dict[Text, Text]] = ...,
        query_auth: bool = ...,
        force_http: bool = ...,
        response_headers: Optional[Dict[Text, Text]] = ...,
        expires_in_absolute: bool = ...,
        version_id: Optional[Any] = ...,
        policy: Optional[Any] = ...,
        reduced_redundancy: bool = ...,
        encrypt_key: bool = ...,
    ): ...
    def send_file(
        self,
        fp,
        headers: Optional[Dict[Text, Text]] = ...,
        cb: Optional[Callable[[int, int], Any]] = ...,
        num_cb: int = ...,
        query_args: Optional[Any] = ...,
        chunked_transfer: bool = ...,
        size: Optional[Any] = ...,
    ): ...
    def should_retry(self, response, chunked_transfer: bool = ...): ...
    def compute_md5(self, fp, size: Optional[Any] = ...): ...
    def set_contents_from_stream(
        self,
        fp,
        headers: Optional[Dict[Text, Text]] = ...,
        replace: bool = ...,
        cb: Optional[Callable[[int, int], Any]] = ...,
        num_cb: int = ...,
        policy: Optional[Any] = ...,
        reduced_redundancy: bool = ...,
        query_args: Optional[Any] = ...,
        size: Optional[Any] = ...,
    ): ...
    def set_contents_from_file(
        self,
        fp,
        headers: Optional[Dict[Text, Text]] = ...,
        replace: bool = ...,
        cb: Optional[Callable[[int, int], Any]] = ...,
        num_cb: int = ...,
        policy: Optional[Any] = ...,
        md5: Optional[Any] = ...,
        reduced_redundancy: bool = ...,
        query_args: Optional[Any] = ...,
        encrypt_key: bool = ...,
        size: Optional[Any] = ...,
        rewind: bool = ...,
    ): ...
    def set_contents_from_filename(
        self,
        filename,
        headers: Optional[Dict[Text, Text]] = ...,
        replace: bool = ...,
        cb: Optional[Callable[[int, int], Any]] = ...,
        num_cb: int = ...,
        policy: Optional[Any] = ...,
        md5: Optional[Any] = ...,
        reduced_redundancy: bool = ...,
        encrypt_key: bool = ...,
    ): ...
    def set_contents_from_string(
        self,
        string_data: Union[Text, bytes],
        headers: Optional[Dict[Text, Text]] = ...,
        replace: bool = ...,
        cb: Optional[Callable[[int, int], Any]] = ...,
        num_cb: int = ...,
        policy: Optional[Any] = ...,
        md5: Optional[Any] = ...,
        reduced_redundancy: bool = ...,
        encrypt_key: bool = ...,
    ) -> None: ...
    def get_file(
        self,
        fp,
        headers: Optional[Dict[Text, Text]] = ...,
        cb: Optional[Callable[[int, int], Any]] = ...,
        num_cb: int = ...,
        torrent: bool = ...,
        version_id: Optional[Any] = ...,
        override_num_retries: Optional[Any] = ...,
        response_headers: Optional[Dict[Text, Text]] = ...,
    ): ...
    def get_torrent_file(
        self, fp, headers: Optional[Dict[Text, Text]] = ..., cb: Optional[Callable[[int, int], Any]] = ..., num_cb: int = ...
    ): ...
    def get_contents_to_file(
        self,
        fp,
        headers: Optional[Dict[Text, Text]] = ...,
        cb: Optional[Callable[[int, int], Any]] = ...,
        num_cb: int = ...,
        torrent: bool = ...,
        version_id: Optional[Any] = ...,
        res_download_handler: Optional[Any] = ...,
        response_headers: Optional[Dict[Text, Text]] = ...,
    ): ...
    def get_contents_to_filename(
        self,
        filename,
        headers: Optional[Dict[Text, Text]] = ...,
        cb: Optional[Callable[[int, int], Any]] = ...,
        num_cb: int = ...,
        torrent: bool = ...,
        version_id: Optional[Any] = ...,
        res_download_handler: Optional[Any] = ...,
        response_headers: Optional[Dict[Text, Text]] = ...,
    ): ...
    @overload
    def get_contents_as_string(
        self,
        headers: Optional[Dict[Text, Text]] = ...,
        cb: Optional[Callable[[int, int], Any]] = ...,
        num_cb: int = ...,
        torrent: bool = ...,
        version_id: Optional[Any] = ...,
        response_headers: Optional[Dict[Text, Text]] = ...,
        encoding: None = ...,
    ) -> bytes: ...
    @overload
    def get_contents_as_string(
        self,
        headers: Optional[Dict[Text, Text]] = ...,
        cb: Optional[Callable[[int, int], Any]] = ...,
        num_cb: int = ...,
        torrent: bool = ...,
        version_id: Optional[Any] = ...,
        response_headers: Optional[Dict[Text, Text]] = ...,
        *, encoding: Text,
    ) -> Text: ...
    def add_email_grant(self, permission, email_address, headers: Optional[Dict[Text, Text]] = ...): ...
    def add_user_grant(
        self, permission, user_id, headers: Optional[Dict[Text, Text]] = ..., display_name: Optional[Any] = ...
    ): ...
    def set_remote_metadata(self, metadata_plus, metadata_minus, preserve_acl, headers: Optional[Dict[Text, Text]] = ...): ...
    def restore(self, days, headers: Optional[Dict[Text, Text]] = ...): ...



# Stubs for lib2to3.pgen2.tokenize (Python 3.6)
# NOTE: Only elements from __all__ are present.

from typing import Callable, Iterable, Iterator, List, Text, Tuple
from lib2to3.pgen2.token import *  # noqa


_Coord = Tuple[int, int]
_TokenEater = Callable[[int, Text, _Coord, _Coord, Text], None]
_TokenInfo = Tuple[int, Text, _Coord, _Coord, Text]


class TokenError(Exception): ...
class StopTokenizing(Exception): ...

def tokenize(readline: Callable[[], Text], tokeneater: _TokenEater = ...) -> None: ...

class Untokenizer:
    tokens: List[Text]
    prev_row: int
    prev_col: int
    def __init__(self) -> None: ...
    def add_whitespace(self, start: _Coord) -> None: ...
    def untokenize(self, iterable: Iterable[_TokenInfo]) -> Text: ...
    def compat(self, token: Tuple[int, Text], iterable: Iterable[_TokenInfo]) -> None: ...

def untokenize(iterable: Iterable[_TokenInfo]) -> Text: ...
def generate_tokens(
    readline: Callable[[], Text]
) -> Iterator[_TokenInfo]: ...



import sys
from typing import Any, Iterable, List, Optional, Protocol, Tuple, Union

class _HasFileno(Protocol):
    def fileno(self) -> int: ...

_FileDescriptor = Union[int, _HasFileno]

EPOLLERR: int
EPOLLET: int
EPOLLHUP: int
EPOLLIN: int
EPOLLMSG: int
EPOLLONESHOT: int
EPOLLOUT: int
EPOLLPRI: int
EPOLLRDBAND: int
EPOLLRDNORM: int
EPOLLWRBAND: int
EPOLLWRNORM: int
EPOLL_RDHUP: int
KQ_EV_ADD: int
KQ_EV_CLEAR: int
KQ_EV_DELETE: int
KQ_EV_DISABLE: int
KQ_EV_ENABLE: int
KQ_EV_EOF: int
KQ_EV_ERROR: int
KQ_EV_FLAG1: int
KQ_EV_ONESHOT: int
KQ_EV_SYSFLAGS: int
KQ_FILTER_AIO: int
KQ_FILTER_NETDEV: int
KQ_FILTER_PROC: int
KQ_FILTER_READ: int
KQ_FILTER_SIGNAL: int
KQ_FILTER_TIMER: int
KQ_FILTER_VNODE: int
KQ_FILTER_WRITE: int
KQ_NOTE_ATTRIB: int
KQ_NOTE_CHILD: int
KQ_NOTE_DELETE: int
KQ_NOTE_EXEC: int
KQ_NOTE_EXIT: int
KQ_NOTE_EXTEND: int
KQ_NOTE_FORK: int
KQ_NOTE_LINK: int
KQ_NOTE_LINKDOWN: int
KQ_NOTE_LINKINV: int
KQ_NOTE_LINKUP: int
KQ_NOTE_LOWAT: int
KQ_NOTE_PCTRLMASK: int
KQ_NOTE_PDATAMASK: int
KQ_NOTE_RENAME: int
KQ_NOTE_REVOKE: int
KQ_NOTE_TRACK: int
KQ_NOTE_TRACKERR: int
KQ_NOTE_WRITE: int
PIPE_BUF: int
POLLERR: int
POLLHUP: int
POLLIN: int
POLLMSG: int
POLLNVAL: int
POLLOUT: int
POLLPRI: int
POLLRDBAND: int
POLLRDNORM: int
POLLWRBAND: int
POLLWRNORM: int

class poll:
    def __init__(self) -> None: ...
    def register(self, fd: _FileDescriptor, eventmask: int = ...) -> None: ...
    def modify(self, fd: _FileDescriptor, eventmask: int) -> None: ...
    def unregister(self, fd: _FileDescriptor) -> None: ...
    def poll(self, timeout: Optional[float] = ...) -> List[Tuple[int, int]]: ...

def select(rlist: Iterable[Any], wlist: Iterable[Any], xlist: Iterable[Any],
           timeout: Optional[float] = ...) -> Tuple[List[Any],
                                                    List[Any],
                                                    List[Any]]: ...

if sys.version_info >= (3, 3):
    error = OSError
else:
    class error(Exception): ...

# BSD only
class kevent(object):
    data: Any
    fflags: int
    filter: int
    flags: int
    ident: int
    udata: Any
    def __init__(self, ident: _FileDescriptor, filter: int = ..., flags: int = ..., fflags: int = ..., data: Any = ..., udata: Any = ...) -> None: ...

# BSD only
class kqueue(object):
    closed: bool
    def __init__(self) -> None: ...
    def close(self) -> None: ...
    def control(self, changelist: Optional[Iterable[kevent]], max_events: int, timeout: float = ...) -> List[kevent]: ...
    def fileno(self) -> int: ...
    @classmethod
    def fromfd(cls, fd: _FileDescriptor) -> kqueue: ...

# Linux only
class epoll(object):
    if sys.version_info >= (3, 3):
        def __init__(self, sizehint: int = ..., flags: int = ...) -> None: ...
    else:
        def __init__(self, sizehint: int = ...) -> None: ...
    if sys.version_info >= (3, 4):
        def __enter__(self) -> epoll: ...
        def __exit__(self, *args: Any) -> None: ...
    def close(self) -> None: ...
    closed: bool
    def fileno(self) -> int: ...
    def register(self, fd: _FileDescriptor, eventmask: int = ...) -> None: ...
    def modify(self, fd: _FileDescriptor, eventmask: int) -> None: ...
    def unregister(self, fd: _FileDescriptor) -> None: ...
    def poll(self, timeout: float = ..., maxevents: int = ...) -> List[Tuple[int, int]]: ...
    @classmethod
    def fromfd(cls, fd: _FileDescriptor) -> epoll: ...

if sys.version_info >= (3, 3):
    # Solaris only
    class devpoll:
        if sys.version_info >= (3, 4):
            def close(self) -> None: ...
            closed: bool
            def fileno(self) -> int: ...
        def register(self, fd: _FileDescriptor, eventmask: int = ...) -> None: ...
        def modify(self, fd: _FileDescriptor, eventmask: int = ...) -> None: ...
        def unregister(self, fd: _FileDescriptor) -> None: ...
        def poll(self, timeout: Optional[float] = ...) -> List[Tuple[int, int]]: ...



from typing import Any

from django.contrib.messages.storage.base import BaseStorage

class FallbackStorage(BaseStorage):
    storage_classes: Any = ...
    storages: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...



# Stubs for flask.wrappers (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Dict, Optional
from werkzeug.exceptions import HTTPException
from werkzeug.routing import Rule
from werkzeug.wrappers import Request as RequestBase, Response as ResponseBase

class JSONMixin:
    @property
    def is_json(self) -> bool: ...
    @property
    def json(self): ...
    def get_json(self, force: bool = ..., silent: bool = ..., cache: bool = ...): ...
    def on_json_loading_failed(self, e: Any) -> None: ...

class Request(RequestBase, JSONMixin):
    url_rule: Optional[Rule] = ...
    view_args: Dict[str, Any] = ...
    routing_exception: Optional[HTTPException] = ...
    # Request is making the max_content_length readonly, where it was not the
    # case in its supertype.
    # We would require something like https://github.com/python/typing/issues/241
    @property
    def max_content_length(self) -> Optional[int]: ...  # type: ignore
    @property
    def endpoint(self) -> Optional[str]: ...
    @property
    def blueprint(self) -> Optional[str]: ...

class Response(ResponseBase, JSONMixin):
    default_mimetype: str = ...
    @property
    def max_cookie_size(self) -> int: ...



"""
Contains all classes and functions to deal with lists, dicts, generators and
iterators in general.
"""
import sys

from jedi._compatibility import force_unicode, is_py3
from jedi.inference import compiled
from jedi.inference import analysis
from jedi.inference.lazy_value import LazyKnownValue, LazyKnownValues, \
    LazyTreeValue
from jedi.inference.helpers import get_int_or_none, is_string, \
    reraise_getitem_errors, SimpleGetItemNotFound
from jedi.inference.utils import safe_property, to_list
from jedi.inference.cache import inference_state_method_cache
from jedi.inference.filters import LazyAttributeOverwrite, publish_method
from jedi.inference.base_value import ValueSet, Value, NO_VALUES, \
    ContextualizedNode, iterate_values, sentinel, \
    LazyValueWrapper
from jedi.parser_utils import get_sync_comp_fors
from jedi.inference.context import CompForContext
from jedi.inference.value.dynamic_arrays import check_array_additions


class IterableMixin(object):
    def py__next__(self, contextualized_node=None):
        return self.py__iter__(contextualized_node)

    def py__stop_iteration_returns(self):
        return ValueSet([compiled.builtin_from_name(self.inference_state, u'None')])

    # At the moment, safe values are simple values like "foo", 1 and not
    # lists/dicts. Therefore as a small speed optimization we can just do the
    # default instead of resolving the lazy wrapped values, that are just
    # doing this in the end as well.
    # This mostly speeds up patterns like `sys.version_info >= (3, 0)` in
    # typeshed.
    if sys.version_info[0] == 2:
        # Python 2...........
        def get_safe_value(self, default=sentinel):
            if default is sentinel:
                raise ValueError("There exists no safe value for value %s" % self)
            return default
    else:
        get_safe_value = Value.get_safe_value


class GeneratorBase(LazyAttributeOverwrite, IterableMixin):
    array_type = None

    def _get_wrapped_value(self):
        instance, = self._get_cls().execute_annotation()
        return instance

    def _get_cls(self):
        generator, = self.inference_state.typing_module.py__getattribute__('Generator')
        return generator

    def py__bool__(self):
        return True

    @publish_method('__iter__')
    def _iter(self, arguments):
        return ValueSet([self])

    @publish_method('send')
    @publish_method('next', python_version_match=2)
    @publish_method('__next__', python_version_match=3)
    def _next(self, arguments):
        return ValueSet.from_sets(lazy_value.infer() for lazy_value in self.py__iter__())

    def py__stop_iteration_returns(self):
        return ValueSet([compiled.builtin_from_name(self.inference_state, u'None')])

    @property
    def name(self):
        return compiled.CompiledValueName(self, 'Generator')

    def get_annotated_class_object(self):
        from jedi.inference.gradual.generics import TupleGenericManager
        gen_values = self.merge_types_of_iterate().py__class__()
        gm = TupleGenericManager((gen_values, NO_VALUES, NO_VALUES))
        return self._get_cls().with_generics(gm)


class Generator(GeneratorBase):
    """Handling of `yield` functions."""
    def __init__(self, inference_state, func_execution_context):
        super(Generator, self).__init__(inference_state)
        self._func_execution_context = func_execution_context

    def py__iter__(self, contextualized_node=None):
        iterators = self._func_execution_context.infer_annotations()
        if iterators:
            return iterators.iterate(contextualized_node)
        return self._func_execution_context.get_yield_lazy_values()

    def py__stop_iteration_returns(self):
        return self._func_execution_context.get_return_values()

    def __repr__(self):
        return "<%s of %s>" % (type(self).__name__, self._func_execution_context)


def comprehension_from_atom(inference_state, value, atom):
    bracket = atom.children[0]
    test_list_comp = atom.children[1]

    if bracket == '{':
        if atom.children[1].children[1] == ':':
            sync_comp_for = test_list_comp.children[3]
            if sync_comp_for.type == 'comp_for':
                sync_comp_for = sync_comp_for.children[1]

            return DictComprehension(
                inference_state,
                value,
                sync_comp_for_node=sync_comp_for,
                key_node=test_list_comp.children[0],
                value_node=test_list_comp.children[2],
            )
        else:
            cls = SetComprehension
    elif bracket == '(':
        cls = GeneratorComprehension
    elif bracket == '[':
        cls = ListComprehension

    sync_comp_for = test_list_comp.children[1]
    if sync_comp_for.type == 'comp_for':
        sync_comp_for = sync_comp_for.children[1]

    return cls(
        inference_state,
        defining_context=value,
        sync_comp_for_node=sync_comp_for,
        entry_node=test_list_comp.children[0],
    )


class ComprehensionMixin(object):
    @inference_state_method_cache()
    def _get_comp_for_context(self, parent_context, comp_for):
        return CompForContext(parent_context, comp_for)

    def _nested(self, comp_fors, parent_context=None):
        comp_for = comp_fors[0]

        is_async = comp_for.parent.type == 'comp_for'

        input_node = comp_for.children[3]
        parent_context = parent_context or self._defining_context
        input_types = parent_context.infer_node(input_node)

        cn = ContextualizedNode(parent_context, input_node)
        iterated = input_types.iterate(cn, is_async=is_async)
        exprlist = comp_for.children[1]
        for i, lazy_value in enumerate(iterated):
            types = lazy_value.infer()
            dct = unpack_tuple_to_dict(parent_context, types, exprlist)
            context = self._get_comp_for_context(
                parent_context,
                comp_for,
            )
            with context.predefine_names(comp_for, dct):
                try:
                    for result in self._nested(comp_fors[1:], context):
                        yield result
                except IndexError:
                    iterated = context.infer_node(self._entry_node)
                    if self.array_type == 'dict':
                        yield iterated, context.infer_node(self._value_node)
                    else:
                        yield iterated

    @inference_state_method_cache(default=[])
    @to_list
    def _iterate(self):
        comp_fors = tuple(get_sync_comp_fors(self._sync_comp_for_node))
        for result in self._nested(comp_fors):
            yield result

    def py__iter__(self, contextualized_node=None):
        for set_ in self._iterate():
            yield LazyKnownValues(set_)

    def __repr__(self):
        return "<%s of %s>" % (type(self).__name__, self._sync_comp_for_node)


class _DictMixin(object):
    def _get_generics(self):
        return tuple(c_set.py__class__() for c_set in self.get_mapping_item_values())


class Sequence(LazyAttributeOverwrite, IterableMixin):
    api_type = u'instance'

    @property
    def name(self):
        return compiled.CompiledValueName(self, self.array_type)

    def _get_generics(self):
        return (self.merge_types_of_iterate().py__class__(),)

    @inference_state_method_cache(default=())
    def _cached_generics(self):
        return self._get_generics()

    def _get_wrapped_value(self):
        from jedi.inference.gradual.base import GenericClass
        from jedi.inference.gradual.generics import TupleGenericManager
        klass = compiled.builtin_from_name(self.inference_state, self.array_type)
        c, = GenericClass(
            klass,
            TupleGenericManager(self._cached_generics())
        ).execute_annotation()
        return c

    def py__bool__(self):
        return None  # We don't know the length, because of appends.

    @safe_property
    def parent(self):
        return self.inference_state.builtins_module

    def py__getitem__(self, index_value_set, contextualized_node):
        if self.array_type == 'dict':
            return self._dict_values()
        return iterate_values(ValueSet([self]))


class _BaseComprehension(ComprehensionMixin):
    def __init__(self, inference_state, defining_context, sync_comp_for_node, entry_node):
        assert sync_comp_for_node.type == 'sync_comp_for'
        super(_BaseComprehension, self).__init__(inference_state)
        self._defining_context = defining_context
        self._sync_comp_for_node = sync_comp_for_node
        self._entry_node = entry_node


class ListComprehension(_BaseComprehension, Sequence):
    array_type = u'list'

    def py__simple_getitem__(self, index):
        if isinstance(index, slice):
            return ValueSet([self])

        all_types = list(self.py__iter__())
        with reraise_getitem_errors(IndexError, TypeError):
            lazy_value = all_types[index]
        return lazy_value.infer()


class SetComprehension(_BaseComprehension, Sequence):
    array_type = u'set'


class GeneratorComprehension(_BaseComprehension, GeneratorBase):
    pass


class _DictKeyMixin(object):
    # TODO merge with _DictMixin?
    def get_mapping_item_values(self):
        return self._dict_keys(), self._dict_values()

    def get_key_values(self):
        # TODO merge with _dict_keys?
        return self._dict_keys()


class DictComprehension(ComprehensionMixin, Sequence, _DictKeyMixin):
    array_type = u'dict'

    def __init__(self, inference_state, defining_context, sync_comp_for_node, key_node, value_node):
        assert sync_comp_for_node.type == 'sync_comp_for'
        super(DictComprehension, self).__init__(inference_state)
        self._defining_context = defining_context
        self._sync_comp_for_node = sync_comp_for_node
        self._entry_node = key_node
        self._value_node = value_node

    def py__iter__(self, contextualized_node=None):
        for keys, values in self._iterate():
            yield LazyKnownValues(keys)

    def py__simple_getitem__(self, index):
        for keys, values in self._iterate():
            for k in keys:
                # Be careful in the future if refactoring, index could be a
                # slice object.
                if k.get_safe_value(default=object()) == index:
                    return values
        raise SimpleGetItemNotFound()

    def _dict_keys(self):
        return ValueSet.from_sets(keys for keys, values in self._iterate())

    def _dict_values(self):
        return ValueSet.from_sets(values for keys, values in self._iterate())

    @publish_method('values')
    def _imitate_values(self, arguments):
        lazy_value = LazyKnownValues(self._dict_values())
        return ValueSet([FakeList(self.inference_state, [lazy_value])])

    @publish_method('items')
    def _imitate_items(self, arguments):
        lazy_values = [
            LazyKnownValue(
                FakeTuple(
                    self.inference_state,
                    [LazyKnownValues(key),
                     LazyKnownValues(value)]
                )
            )
            for key, value in self._iterate()
        ]

        return ValueSet([FakeList(self.inference_state, lazy_values)])

    def exact_key_items(self):
        # NOTE: A smarter thing can probably done here to achieve better
        # completions, but at least like this jedi doesn't crash
        return []


class SequenceLiteralValue(Sequence):
    _TUPLE_LIKE = 'testlist_star_expr', 'testlist', 'subscriptlist'
    mapping = {'(': u'tuple',
               '[': u'list',
               '{': u'set'}

    def __init__(self, inference_state, defining_context, atom):
        super(SequenceLiteralValue, self).__init__(inference_state)
        self.atom = atom
        self._defining_context = defining_context

        if self.atom.type in self._TUPLE_LIKE:
            self.array_type = u'tuple'
        else:
            self.array_type = SequenceLiteralValue.mapping[atom.children[0]]
            """The builtin name of the array (list, set, tuple or dict)."""

    def _get_generics(self):
        if self.array_type == u'tuple':
            return tuple(x.infer().py__class__() for x in self.py__iter__())
        return super(SequenceLiteralValue, self)._get_generics()

    def py__simple_getitem__(self, index):
        """Here the index is an int/str. Raises IndexError/KeyError."""
        if isinstance(index, slice):
            return ValueSet([self])
        else:
            with reraise_getitem_errors(TypeError, KeyError, IndexError):
                node = self.get_tree_entries()[index]
            return self._defining_context.infer_node(node)

    def py__iter__(self, contextualized_node=None):
        """
        While values returns the possible values for any array field, this
        function returns the value for a certain index.
        """
        for node in self.get_tree_entries():
            if node == ':' or node.type == 'subscript':
                # TODO this should probably use at least part of the code
                #      of infer_subscript_list.
                yield LazyKnownValue(Slice(self._defining_context, None, None, None))
            else:
                yield LazyTreeValue(self._defining_context, node)
        for addition in check_array_additions(self._defining_context, self):
            yield addition

    def py__len__(self):
        # This function is not really used often. It's more of a try.
        return len(self.get_tree_entries())

    def get_tree_entries(self):
        c = self.atom.children

        if self.atom.type in self._TUPLE_LIKE:
            return c[::2]

        array_node = c[1]
        if array_node in (']', '}', ')'):
            return []  # Direct closing bracket, doesn't contain items.

        if array_node.type == 'testlist_comp':
            # filter out (for now) pep 448 single-star unpacking
            return [value for value in array_node.children[::2]
                    if value.type != "star_expr"]
        elif array_node.type == 'dictorsetmaker':
            kv = []
            iterator = iter(array_node.children)
            for key in iterator:
                if key == "**":
                    # dict with pep 448 double-star unpacking
                    # for now ignoring the values imported by **
                    next(iterator)
                    next(iterator, None)  # Possible comma.
                else:
                    op = next(iterator, None)
                    if op is None or op == ',':
                        if key.type == "star_expr":
                            # pep 448 single-star unpacking
                            # for now ignoring values imported by *
                            pass
                        else:
                            kv.append(key)  # A set.
                    else:
                        assert op == ':'  # A dict.
                        kv.append((key, next(iterator)))
                        next(iterator, None)  # Possible comma.
            return kv
        else:
            if array_node.type == "star_expr":
                # pep 448 single-star unpacking
                # for now ignoring values imported by *
                return []
            else:
                return [array_node]

    def exact_key_items(self):
        """
        Returns a generator of tuples like dict.items(), where the key is
        resolved (as a string) and the values are still lazy values.
        """
        for key_node, value in self.get_tree_entries():
            for key in self._defining_context.infer_node(key_node):
                if is_string(key):
                    yield key.get_safe_value(), LazyTreeValue(self._defining_context, value)

    def __repr__(self):
        return "<%s of %s>" % (self.__class__.__name__, self.atom)


class DictLiteralValue(_DictMixin, SequenceLiteralValue, _DictKeyMixin):
    array_type = u'dict'

    def __init__(self, inference_state, defining_context, atom):
        super(SequenceLiteralValue, self).__init__(inference_state)
        self._defining_context = defining_context
        self.atom = atom

    def py__simple_getitem__(self, index):
        """Here the index is an int/str. Raises IndexError/KeyError."""
        compiled_value_index = compiled.create_simple_object(self.inference_state, index)
        for key, value in self.get_tree_entries():
            for k in self._defining_context.infer_node(key):
                for key_v in k.execute_operation(compiled_value_index, u'=='):
                    if key_v.get_safe_value():
                        return self._defining_context.infer_node(value)
        raise SimpleGetItemNotFound('No key found in dictionary %s.' % self)

    def py__iter__(self, contextualized_node=None):
        """
        While values returns the possible values for any array field, this
        function returns the value for a certain index.
        """
        # Get keys.
        types = NO_VALUES
        for k, _ in self.get_tree_entries():
            types |= self._defining_context.infer_node(k)
        # We don't know which dict index comes first, therefore always
        # yield all the types.
        for _ in types:
            yield LazyKnownValues(types)

    @publish_method('values')
    def _imitate_values(self, arguments):
        lazy_value = LazyKnownValues(self._dict_values())
        return ValueSet([FakeList(self.inference_state, [lazy_value])])

    @publish_method('items')
    def _imitate_items(self, arguments):
        lazy_values = [
            LazyKnownValue(FakeTuple(
                self.inference_state,
                (LazyTreeValue(self._defining_context, key_node),
                 LazyTreeValue(self._defining_context, value_node))
            )) for key_node, value_node in self.get_tree_entries()
        ]

        return ValueSet([FakeList(self.inference_state, lazy_values)])

    def _dict_values(self):
        return ValueSet.from_sets(
            self._defining_context.infer_node(v)
            for k, v in self.get_tree_entries()
        )

    def _dict_keys(self):
        return ValueSet.from_sets(
            self._defining_context.infer_node(k)
            for k, v in self.get_tree_entries()
        )


class _FakeSequence(Sequence):
    def __init__(self, inference_state, lazy_value_list):
        """
        type should be one of "tuple", "list"
        """
        super(_FakeSequence, self).__init__(inference_state)
        self._lazy_value_list = lazy_value_list

    def py__simple_getitem__(self, index):
        if isinstance(index, slice):
            return ValueSet([self])

        with reraise_getitem_errors(IndexError, TypeError):
            lazy_value = self._lazy_value_list[index]
        return lazy_value.infer()

    def py__iter__(self, contextualized_node=None):
        return self._lazy_value_list

    def py__bool__(self):
        return bool(len(self._lazy_value_list))

    def __repr__(self):
        return "<%s of %s>" % (type(self).__name__, self._lazy_value_list)


class FakeTuple(_FakeSequence):
    array_type = u'tuple'


class FakeList(_FakeSequence):
    array_type = u'tuple'


class FakeDict(_DictMixin, Sequence, _DictKeyMixin):
    array_type = u'dict'

    def __init__(self, inference_state, dct):
        super(FakeDict, self).__init__(inference_state)
        self._dct = dct

    def py__iter__(self, contextualized_node=None):
        for key in self._dct:
            yield LazyKnownValue(compiled.create_simple_object(self.inference_state, key))

    def py__simple_getitem__(self, index):
        if is_py3 and self.inference_state.environment.version_info.major == 2:
            # In Python 2 bytes and unicode compare.
            if isinstance(index, bytes):
                index_unicode = force_unicode(index)
                try:
                    return self._dct[index_unicode].infer()
                except KeyError:
                    pass
            elif isinstance(index, str):
                index_bytes = index.encode('utf-8')
                try:
                    return self._dct[index_bytes].infer()
                except KeyError:
                    pass

        with reraise_getitem_errors(KeyError, TypeError):
            lazy_value = self._dct[index]
        return lazy_value.infer()

    @publish_method('values')
    def _values(self, arguments):
        return ValueSet([FakeTuple(
            self.inference_state,
            [LazyKnownValues(self._dict_values())]
        )])

    def _dict_values(self):
        return ValueSet.from_sets(lazy_value.infer() for lazy_value in self._dct.values())

    def _dict_keys(self):
        return ValueSet.from_sets(lazy_value.infer() for lazy_value in self.py__iter__())

    def exact_key_items(self):
        return self._dct.items()

    def __repr__(self):
        return '<%s: %s>' % (self.__class__.__name__, self._dct)


class MergedArray(Sequence):
    def __init__(self, inference_state, arrays):
        super(MergedArray, self).__init__(inference_state)
        self.array_type = arrays[-1].array_type
        self._arrays = arrays

    def py__iter__(self, contextualized_node=None):
        for array in self._arrays:
            for lazy_value in array.py__iter__():
                yield lazy_value

    def py__simple_getitem__(self, index):
        return ValueSet.from_sets(lazy_value.infer() for lazy_value in self.py__iter__())


def unpack_tuple_to_dict(context, types, exprlist):
    """
    Unpacking tuple assignments in for statements and expr_stmts.
    """
    if exprlist.type == 'name':
        return {exprlist.value: types}
    elif exprlist.type == 'atom' and exprlist.children[0] in ('(', '['):
        return unpack_tuple_to_dict(context, types, exprlist.children[1])
    elif exprlist.type in ('testlist', 'testlist_comp', 'exprlist',
                           'testlist_star_expr'):
        dct = {}
        parts = iter(exprlist.children[::2])
        n = 0
        for lazy_value in types.iterate(ContextualizedNode(context, exprlist)):
            n += 1
            try:
                part = next(parts)
            except StopIteration:
                analysis.add(context, 'value-error-too-many-values', part,
                             message="ValueError: too many values to unpack (expected %s)" % n)
            else:
                dct.update(unpack_tuple_to_dict(context, lazy_value.infer(), part))
        has_parts = next(parts, None)
        if types and has_parts is not None:
            analysis.add(context, 'value-error-too-few-values', has_parts,
                         message="ValueError: need more than %s values to unpack" % n)
        return dct
    elif exprlist.type == 'power' or exprlist.type == 'atom_expr':
        # Something like ``arr[x], var = ...``.
        # This is something that is not yet supported, would also be difficult
        # to write into a dict.
        return {}
    elif exprlist.type == 'star_expr':  # `a, *b, c = x` type unpackings
        # Currently we're not supporting them.
        return {}
    raise NotImplementedError


class Slice(LazyValueWrapper):
    def __init__(self, python_context, start, stop, step):
        self.inference_state = python_context.inference_state
        self._context = python_context
        # All of them are either a Precedence or None.
        self._start = start
        self._stop = stop
        self._step = step

    def _get_wrapped_value(self):
        value = compiled.builtin_from_name(self._context.inference_state, 'slice')
        slice_value, = value.execute_with_values()
        return slice_value

    def get_safe_value(self, default=sentinel):
        """
        Imitate CompiledValue.obj behavior and return a ``builtin.slice()``
        object.
        """
        def get(element):
            if element is None:
                return None

            result = self._context.infer_node(element)
            if len(result) != 1:
                # For simplicity, we want slices to be clear defined with just
                # one type.  Otherwise we will return an empty slice object.
                raise IndexError

            value, = result
            return get_int_or_none(value)

        try:
            return slice(get(self._start), get(self._stop), get(self._step))
        except IndexError:
            return slice(None, None, None)



from typing import Any, List, Optional, Sequence, Union

from django.contrib.messages.storage.base import BaseStorage
from django.http.request import HttpRequest

class SessionStorage(BaseStorage):
    session_key: str = ...
    def __init__(self, request: HttpRequest, *args: Any, **kwargs: Any) -> None: ...
    def serialize_messages(self, messages: Sequence[Any]) -> str: ...
    def deserialize_messages(self, data: Optional[Union[List[Any], str]]) -> Optional[List[Any]]: ...



from typing import Any, List, Optional, Tuple

from .message import (
    BadHeaderError as BadHeaderError,
    DEFAULT_ATTACHMENT_MIME_TYPE as DEFAULT_ATTACHMENT_MIME_TYPE,
    EmailMessage as EmailMessage,
    EmailMultiAlternatives as EmailMultiAlternatives,
    SafeMIMEMultipart as SafeMIMEMultipart,
    SafeMIMEText as SafeMIMEText,
    forbid_multi_line_headers as forbid_multi_line_headers,
)
from .utils import CachedDnsName as CachedDnsName, DNS_NAME as DNS_NAME

def get_connection(backend: Optional[str] = ..., fail_silently: bool = ..., **kwds: Any) -> Any: ...
def send_mail(
    subject: str,
    message: str,
    from_email: Optional[str],
    recipient_list: List[str],
    fail_silently: bool = ...,
    auth_user: Optional[str] = ...,
    auth_password: Optional[str] = ...,
    connection: Optional[Any] = ...,
    html_message: Optional[str] = ...,
) -> int: ...
def send_mass_mail(
    datatuple: List[Tuple[str, str, str, List[str]]],
    fail_silently: bool = ...,
    auth_user: Optional[str] = ...,
    auth_password: Optional[str] = ...,
    connection: Optional[Any] = ...,
) -> int: ...
def mail_admins(
    subject: str,
    message: str,
    fail_silently: bool = ...,
    connection: Optional[Any] = ...,
    html_message: Optional[str] = ...,
) -> None: ...
def mail_managers(
    subject: str,
    message: str,
    fail_silently: bool = ...,
    connection: Optional[Any] = ...,
    html_message: Optional[str] = ...,
) -> None: ...

outbox: List[EmailMessage] = ...



from typing import Any, Callable, Dict, Optional, Tuple, Union, Sequence

color_names: Any
foreground: Any
background: Any
RESET: str
opt_dict: Any

def colorize(text: Optional[str] = ..., opts: Sequence[str] = ..., **kwargs: Any) -> str: ...
def make_style(opts: Tuple = ..., **kwargs: Any) -> Callable: ...

NOCOLOR_PALETTE: str
DARK_PALETTE: str
LIGHT_PALETTE: str
PALETTES: Any
DEFAULT_PALETTE: str = ...

def parse_color_setting(config_string: str) -> Optional[Dict[str, Dict[str, Union[Tuple[str], str]]]]: ...



import six.moves.urllib.error as error
import six.moves.urllib.parse as parse
import six.moves.urllib.request as request
import six.moves.urllib.response as response
import six.moves.urllib.robotparser as robotparser



from typing import Callable, List, Union, Iterator, Tuple, Optional, Any, IO, NamedTuple, Dict

import sys
import types

from opcode import (hasconst as hasconst, hasname as hasname, hasjrel as hasjrel,
                    hasjabs as hasjabs, haslocal as haslocal, hascompare as hascompare,
                    hasfree as hasfree, cmp_op as cmp_op, opname as opname, opmap as opmap,
                    HAVE_ARGUMENT as HAVE_ARGUMENT, EXTENDED_ARG as EXTENDED_ARG)

if sys.version_info >= (3, 4):
    from opcode import stack_effect as stack_effect

if sys.version_info >= (3, 6):
    from opcode import hasnargs as hasnargs

# Strictly this should not have to include Callable, but mypy doesn't use FunctionType
# for functions (python/mypy#3171)
_have_code = Union[types.MethodType, types.FunctionType, types.CodeType, type, Callable[..., Any]]
_have_code_or_string = Union[_have_code, str, bytes]


if sys.version_info >= (3, 4):
    class Instruction(NamedTuple):
        opname: str
        opcode: int
        arg: Optional[int]
        argval: Any
        argrepr: str
        offset: int
        starts_line: Optional[int]
        is_jump_target: bool

    class Bytecode:
        codeobj: types.CodeType
        first_line: int
        def __init__(self, x: _have_code_or_string, *, first_line: Optional[int] = ...,
                     current_offset: Optional[int] = ...) -> None: ...
        def __iter__(self) -> Iterator[Instruction]: ...
        def __repr__(self) -> str: ...
        def info(self) -> str: ...
        def dis(self) -> str: ...

        @classmethod
        def from_traceback(cls, tb: types.TracebackType) -> Bytecode: ...


COMPILER_FLAG_NAMES: Dict[int, str]


def findlabels(code: _have_code) -> List[int]: ...
def findlinestarts(code: _have_code) -> Iterator[Tuple[int, int]]: ...

if sys.version_info >= (3, 0):
    def pretty_flags(flags: int) -> str: ...
    def code_info(x: _have_code_or_string) -> str: ...

if sys.version_info >= (3, 4):
    def dis(x: _have_code_or_string = ..., *, file: Optional[IO[str]] = ...) -> None: ...
    def distb(tb: Optional[types.TracebackType] = ..., *, file: Optional[IO[str]] = ...) -> None: ...
    def disassemble(co: _have_code, lasti: int = ..., *, file: Optional[IO[str]] = ...) -> None: ...
    def disco(co: _have_code, lasti: int = ..., *, file: Optional[IO[str]] = ...) -> None: ...
    def show_code(co: _have_code, *, file: Optional[IO[str]] = ...) -> None: ...

    def get_instructions(x: _have_code, *, first_line: Optional[int] = ...) -> Iterator[Instruction]: ...
else:
    def dis(x: _have_code_or_string = ...) -> None: ...
    def distb(tb: types.TracebackType = ...) -> None: ...
    def disassemble(co: _have_code, lasti: int = ...) -> None: ...
    def disco(co: _have_code, lasti: int = ...) -> None: ...

    if sys.version_info >= (3, 0):
        def show_code(co: _have_code) -> None: ...



# -*- coding: utf-8 -*-
"""
    click
    ~~~~~

    Click is a simple Python module that wraps the stdlib's optparse to make
    writing command line scripts fun.  Unlike other modules, it's based around
    a simple API that does not come with too much magic and is composable.

    In case optparse ever gets removed from the stdlib, it will be shipped by
    this module.

    :copyright: (c) 2014 by Armin Ronacher.
    :license: BSD, see LICENSE for more details.
"""

# Core classes
from .core import (
    Context as Context,
    BaseCommand as BaseCommand,
    Command as Command,
    MultiCommand as MultiCommand,
    Group as Group,
    CommandCollection as CommandCollection,
    Parameter as Parameter,
    Option as Option,
    Argument as Argument,
)

# Globals
from .globals import get_current_context as get_current_context

# Decorators
from .decorators import (
    pass_context as pass_context,
    pass_obj as pass_obj,
    make_pass_decorator as make_pass_decorator,
    command as command,
    group as group,
    argument as argument,
    option as option,
    confirmation_option as confirmation_option,
    password_option as password_option,
    version_option as version_option,
    help_option as help_option,
)

# Types
from .types import (
    ParamType as ParamType,
    File as File,
    FloatRange as FloatRange,
    DateTime as DateTime,
    Path as Path,
    Choice as Choice,
    IntRange as IntRange,
    Tuple as Tuple,
    STRING as STRING,
    INT as INT,
    FLOAT as FLOAT,
    BOOL as BOOL,
    UUID as UUID,
    UNPROCESSED as UNPROCESSED,
)

# Utilities
from .utils import (
    echo as echo,
    get_binary_stream as get_binary_stream,
    get_text_stream as get_text_stream,
    open_file as open_file,
    format_filename as format_filename,
    get_app_dir as get_app_dir,
    get_os_args as get_os_args,
)

# Terminal functions
from .termui import (
    prompt as prompt,
    confirm as confirm,
    get_terminal_size as get_terminal_size,
    echo_via_pager as echo_via_pager,
    progressbar as progressbar,
    clear as clear,
    style as style,
    unstyle as unstyle,
    secho as secho,
    edit as edit,
    launch as launch,
    getchar as getchar,
    pause as pause,
)

# Exceptions
from .exceptions import (
    ClickException as ClickException,
    UsageError as UsageError,
    BadParameter as BadParameter,
    FileError as FileError,
    Abort as Abort,
    NoSuchOption as NoSuchOption,
    BadOptionUsage as BadOptionUsage,
    BadArgumentUsage as BadArgumentUsage,
    MissingParameter as MissingParameter,
)

# Formatting
from .formatting import HelpFormatter as HelpFormatter, wrap_text as wrap_text

# Parsing
from .parser import OptionParser as OptionParser

# Controls if click should emit the warning about the use of unicode
# literals.
disable_unicode_literals_warning: bool


__version__: str



from google.protobuf.internal.containers import RepeatedCompositeFieldContainer, RepeatedScalarFieldContainer
from google.protobuf.message import Message
from google.protobuf.unittest_import_pb2 import ImportEnum, ImportMessage
from google.protobuf.unittest_import_public_pb2 import PublicImportMessage
from typing import Iterable, List, Mapping, MutableMapping, Optional, Text, Tuple, cast

class ForeignEnum(int):
    @classmethod
    def Name(cls, number: int) -> bytes: ...
    @classmethod
    def Value(cls, name: bytes) -> ForeignEnum: ...
    @classmethod
    def keys(cls) -> List[bytes]: ...
    @classmethod
    def values(cls) -> List[ForeignEnum]: ...
    @classmethod
    def items(cls) -> List[Tuple[bytes, ForeignEnum]]: ...

FOREIGN_FOO: ForeignEnum
FOREIGN_BAR: ForeignEnum
FOREIGN_BAZ: ForeignEnum

class TestEnumWithDupValue(int):
    @classmethod
    def Name(cls, number: int) -> bytes: ...
    @classmethod
    def Value(cls, name: bytes) -> TestEnumWithDupValue: ...
    @classmethod
    def keys(cls) -> List[bytes]: ...
    @classmethod
    def values(cls) -> List[TestEnumWithDupValue]: ...
    @classmethod
    def items(cls) -> List[Tuple[bytes, TestEnumWithDupValue]]: ...

FOO1: TestEnumWithDupValue
BAR1: TestEnumWithDupValue
BAZ: TestEnumWithDupValue
FOO2: TestEnumWithDupValue
BAR2: TestEnumWithDupValue

class TestSparseEnum(int):
    @classmethod
    def Name(cls, number: int) -> bytes: ...
    @classmethod
    def Value(cls, name: bytes) -> TestSparseEnum: ...
    @classmethod
    def keys(cls) -> List[bytes]: ...
    @classmethod
    def values(cls) -> List[TestSparseEnum]: ...
    @classmethod
    def items(cls) -> List[Tuple[bytes, TestSparseEnum]]: ...

SPARSE_A: TestSparseEnum
SPARSE_B: TestSparseEnum
SPARSE_C: TestSparseEnum
SPARSE_D: TestSparseEnum
SPARSE_E: TestSparseEnum
SPARSE_F: TestSparseEnum
SPARSE_G: TestSparseEnum

class TestAllTypes(Message):
    class NestedEnum(int):
        @classmethod
        def Name(cls, number: int) -> bytes: ...
        @classmethod
        def Value(cls, name: bytes) -> TestAllTypes.NestedEnum: ...
        @classmethod
        def keys(cls) -> List[bytes]: ...
        @classmethod
        def values(cls) -> List[TestAllTypes.NestedEnum]: ...
        @classmethod
        def items(cls) -> List[Tuple[bytes, TestAllTypes.NestedEnum]]: ...
    FOO: TestAllTypes.NestedEnum
    BAR: TestAllTypes.NestedEnum
    BAZ: TestAllTypes.NestedEnum
    NEG: TestAllTypes.NestedEnum
    class NestedMessage(Message):
        bb: int
        def __init__(self, bb: Optional[int] = ...) -> None: ...
    class OptionalGroup(Message):
        a: int
        def __init__(self, a: Optional[int] = ...) -> None: ...
    class RepeatedGroup(Message):
        a: int
        def __init__(self, a: Optional[int] = ...) -> None: ...
    optional_int32: int
    optional_int64: int
    optional_uint32: int
    optional_uint64: int
    optional_sint32: int
    optional_sint64: int
    optional_fixed32: int
    optional_fixed64: int
    optional_sfixed32: int
    optional_sfixed64: int
    optional_float: float
    optional_double: float
    optional_bool: bool
    optional_string: Text
    optional_bytes: bytes
    optional_nested_enum: TestAllTypes.NestedEnum
    optional_foreign_enum: ForeignEnum
    optional_import_enum: ImportEnum
    optional_string_piece: Text
    optional_cord: Text
    repeated_int32: RepeatedScalarFieldContainer[int]
    repeated_int64: RepeatedScalarFieldContainer[int]
    repeated_uint32: RepeatedScalarFieldContainer[int]
    repeated_uint64: RepeatedScalarFieldContainer[int]
    repeated_sint32: RepeatedScalarFieldContainer[int]
    repeated_sint64: RepeatedScalarFieldContainer[int]
    repeated_fixed32: RepeatedScalarFieldContainer[int]
    repeated_fixed64: RepeatedScalarFieldContainer[int]
    repeated_sfixed32: RepeatedScalarFieldContainer[int]
    repeated_sfixed64: RepeatedScalarFieldContainer[int]
    repeated_float: RepeatedScalarFieldContainer[float]
    repeated_double: RepeatedScalarFieldContainer[float]
    repeated_bool: RepeatedScalarFieldContainer[bool]
    repeated_string: RepeatedScalarFieldContainer[Text]
    repeated_bytes: RepeatedScalarFieldContainer[bytes]
    repeated_nested_enum: RepeatedScalarFieldContainer[TestAllTypes.NestedEnum]
    repeated_foreign_enum: RepeatedScalarFieldContainer[ForeignEnum]
    repeated_import_enum: RepeatedScalarFieldContainer[ImportEnum]
    repeated_string_piece: RepeatedScalarFieldContainer[Text]
    repeated_cord: RepeatedScalarFieldContainer[Text]
    default_int32: int
    default_int64: int
    default_uint32: int
    default_uint64: int
    default_sint32: int
    default_sint64: int
    default_fixed32: int
    default_fixed64: int
    default_sfixed32: int
    default_sfixed64: int
    default_float: float
    default_double: float
    default_bool: bool
    default_string: Text
    default_bytes: bytes
    default_nested_enum: TestAllTypes.NestedEnum
    default_foreign_enum: ForeignEnum
    default_import_enum: ImportEnum
    default_string_piece: Text
    default_cord: Text
    oneof_uint32: int
    oneof_string: Text
    oneof_bytes: bytes
    @property
    def optionalgroup(self) -> TestAllTypes.OptionalGroup: ...
    @property
    def optional_nested_message(self) -> TestAllTypes.NestedMessage: ...
    @property
    def optional_foreign_message(self) -> ForeignMessage: ...
    @property
    def optional_import_message(self) -> ImportMessage: ...
    @property
    def optional_public_import_message(self) -> PublicImportMessage: ...
    @property
    def optional_lazy_message(self) -> TestAllTypes.NestedMessage: ...
    @property
    def repeatedgroup(self) -> RepeatedCompositeFieldContainer[TestAllTypes.RepeatedGroup]: ...
    @property
    def repeated_nested_message(self) -> RepeatedCompositeFieldContainer[TestAllTypes.NestedMessage]: ...
    @property
    def repeated_foreign_message(self) -> RepeatedCompositeFieldContainer[ForeignMessage]: ...
    @property
    def repeated_import_message(self) -> RepeatedCompositeFieldContainer[ImportMessage]: ...
    @property
    def repeated_lazy_message(self) -> RepeatedCompositeFieldContainer[TestAllTypes.NestedMessage]: ...
    @property
    def oneof_nested_message(self) -> TestAllTypes.NestedMessage: ...
    def __init__(
        self,
        optional_int32: Optional[int] = ...,
        optional_int64: Optional[int] = ...,
        optional_uint32: Optional[int] = ...,
        optional_uint64: Optional[int] = ...,
        optional_sint32: Optional[int] = ...,
        optional_sint64: Optional[int] = ...,
        optional_fixed32: Optional[int] = ...,
        optional_fixed64: Optional[int] = ...,
        optional_sfixed32: Optional[int] = ...,
        optional_sfixed64: Optional[int] = ...,
        optional_float: Optional[float] = ...,
        optional_double: Optional[float] = ...,
        optional_bool: Optional[bool] = ...,
        optional_string: Optional[Text] = ...,
        optional_bytes: Optional[bytes] = ...,
        optionalgroup: Optional[TestAllTypes.OptionalGroup] = ...,
        optional_nested_message: Optional[TestAllTypes.NestedMessage] = ...,
        optional_foreign_message: Optional[ForeignMessage] = ...,
        optional_import_message: Optional[ImportMessage] = ...,
        optional_nested_enum: Optional[TestAllTypes.NestedEnum] = ...,
        optional_foreign_enum: Optional[ForeignEnum] = ...,
        optional_import_enum: Optional[ImportEnum] = ...,
        optional_string_piece: Optional[Text] = ...,
        optional_cord: Optional[Text] = ...,
        optional_public_import_message: Optional[PublicImportMessage] = ...,
        optional_lazy_message: Optional[TestAllTypes.NestedMessage] = ...,
        repeated_int32: Optional[Iterable[int]] = ...,
        repeated_int64: Optional[Iterable[int]] = ...,
        repeated_uint32: Optional[Iterable[int]] = ...,
        repeated_uint64: Optional[Iterable[int]] = ...,
        repeated_sint32: Optional[Iterable[int]] = ...,
        repeated_sint64: Optional[Iterable[int]] = ...,
        repeated_fixed32: Optional[Iterable[int]] = ...,
        repeated_fixed64: Optional[Iterable[int]] = ...,
        repeated_sfixed32: Optional[Iterable[int]] = ...,
        repeated_sfixed64: Optional[Iterable[int]] = ...,
        repeated_float: Optional[Iterable[float]] = ...,
        repeated_double: Optional[Iterable[float]] = ...,
        repeated_bool: Optional[Iterable[bool]] = ...,
        repeated_string: Optional[Iterable[Text]] = ...,
        repeated_bytes: Optional[Iterable[bytes]] = ...,
        repeatedgroup: Optional[Iterable[TestAllTypes.RepeatedGroup]] = ...,
        repeated_nested_message: Optional[Iterable[TestAllTypes.NestedMessage]] = ...,
        repeated_foreign_message: Optional[Iterable[ForeignMessage]] = ...,
        repeated_import_message: Optional[Iterable[ImportMessage]] = ...,
        repeated_nested_enum: Optional[Iterable[TestAllTypes.NestedEnum]] = ...,
        repeated_foreign_enum: Optional[Iterable[ForeignEnum]] = ...,
        repeated_import_enum: Optional[Iterable[ImportEnum]] = ...,
        repeated_string_piece: Optional[Iterable[Text]] = ...,
        repeated_cord: Optional[Iterable[Text]] = ...,
        repeated_lazy_message: Optional[Iterable[TestAllTypes.NestedMessage]] = ...,
        default_int32: Optional[int] = ...,
        default_int64: Optional[int] = ...,
        default_uint32: Optional[int] = ...,
        default_uint64: Optional[int] = ...,
        default_sint32: Optional[int] = ...,
        default_sint64: Optional[int] = ...,
        default_fixed32: Optional[int] = ...,
        default_fixed64: Optional[int] = ...,
        default_sfixed32: Optional[int] = ...,
        default_sfixed64: Optional[int] = ...,
        default_float: Optional[float] = ...,
        default_double: Optional[float] = ...,
        default_bool: Optional[bool] = ...,
        default_string: Optional[Text] = ...,
        default_bytes: Optional[bytes] = ...,
        default_nested_enum: Optional[TestAllTypes.NestedEnum] = ...,
        default_foreign_enum: Optional[ForeignEnum] = ...,
        default_import_enum: Optional[ImportEnum] = ...,
        default_string_piece: Optional[Text] = ...,
        default_cord: Optional[Text] = ...,
        oneof_uint32: Optional[int] = ...,
        oneof_nested_message: Optional[TestAllTypes.NestedMessage] = ...,
        oneof_string: Optional[Text] = ...,
        oneof_bytes: Optional[bytes] = ...,
    ) -> None: ...

class NestedTestAllTypes(Message):
    @property
    def child(self) -> NestedTestAllTypes: ...
    @property
    def payload(self) -> TestAllTypes: ...
    @property
    def repeated_child(self) -> RepeatedCompositeFieldContainer[NestedTestAllTypes]: ...
    def __init__(
        self,
        child: Optional[NestedTestAllTypes] = ...,
        payload: Optional[TestAllTypes] = ...,
        repeated_child: Optional[Iterable[NestedTestAllTypes]] = ...,
    ) -> None: ...

class TestDeprecatedFields(Message):
    deprecated_int32: int
    deprecated_int32_in_oneof: int
    def __init__(self, deprecated_int32: Optional[int] = ..., deprecated_int32_in_oneof: Optional[int] = ...) -> None: ...

class TestDeprecatedMessage(Message):
    def __init__(self,) -> None: ...

class ForeignMessage(Message):
    c: int
    d: int
    def __init__(self, c: Optional[int] = ..., d: Optional[int] = ...) -> None: ...

class TestReservedFields(Message):
    def __init__(self,) -> None: ...

class TestAllExtensions(Message):
    def __init__(self,) -> None: ...

class OptionalGroup_extension(Message):
    a: int
    def __init__(self, a: Optional[int] = ...) -> None: ...

class RepeatedGroup_extension(Message):
    a: int
    def __init__(self, a: Optional[int] = ...) -> None: ...

class TestGroup(Message):
    class OptionalGroup(Message):
        a: int
        def __init__(self, a: Optional[int] = ...) -> None: ...
    optional_foreign_enum: ForeignEnum
    @property
    def optionalgroup(self) -> TestGroup.OptionalGroup: ...
    def __init__(
        self, optionalgroup: Optional[TestGroup.OptionalGroup] = ..., optional_foreign_enum: Optional[ForeignEnum] = ...
    ) -> None: ...

class TestGroupExtension(Message):
    def __init__(self,) -> None: ...

class TestNestedExtension(Message):
    class OptionalGroup_extension(Message):
        a: int
        def __init__(self, a: Optional[int] = ...) -> None: ...
    def __init__(self,) -> None: ...

class TestRequired(Message):
    a: int
    dummy2: int
    b: int
    dummy4: int
    dummy5: int
    dummy6: int
    dummy7: int
    dummy8: int
    dummy9: int
    dummy10: int
    dummy11: int
    dummy12: int
    dummy13: int
    dummy14: int
    dummy15: int
    dummy16: int
    dummy17: int
    dummy18: int
    dummy19: int
    dummy20: int
    dummy21: int
    dummy22: int
    dummy23: int
    dummy24: int
    dummy25: int
    dummy26: int
    dummy27: int
    dummy28: int
    dummy29: int
    dummy30: int
    dummy31: int
    dummy32: int
    c: int
    def __init__(
        self,
        a: int,
        b: int,
        c: int,
        dummy2: Optional[int] = ...,
        dummy4: Optional[int] = ...,
        dummy5: Optional[int] = ...,
        dummy6: Optional[int] = ...,
        dummy7: Optional[int] = ...,
        dummy8: Optional[int] = ...,
        dummy9: Optional[int] = ...,
        dummy10: Optional[int] = ...,
        dummy11: Optional[int] = ...,
        dummy12: Optional[int] = ...,
        dummy13: Optional[int] = ...,
        dummy14: Optional[int] = ...,
        dummy15: Optional[int] = ...,
        dummy16: Optional[int] = ...,
        dummy17: Optional[int] = ...,
        dummy18: Optional[int] = ...,
        dummy19: Optional[int] = ...,
        dummy20: Optional[int] = ...,
        dummy21: Optional[int] = ...,
        dummy22: Optional[int] = ...,
        dummy23: Optional[int] = ...,
        dummy24: Optional[int] = ...,
        dummy25: Optional[int] = ...,
        dummy26: Optional[int] = ...,
        dummy27: Optional[int] = ...,
        dummy28: Optional[int] = ...,
        dummy29: Optional[int] = ...,
        dummy30: Optional[int] = ...,
        dummy31: Optional[int] = ...,
        dummy32: Optional[int] = ...,
    ) -> None: ...

class TestRequiredForeign(Message):
    dummy: int
    @property
    def optional_message(self) -> TestRequired: ...
    @property
    def repeated_message(self) -> RepeatedCompositeFieldContainer[TestRequired]: ...
    def __init__(
        self,
        optional_message: Optional[TestRequired] = ...,
        repeated_message: Optional[Iterable[TestRequired]] = ...,
        dummy: Optional[int] = ...,
    ) -> None: ...

class TestRequiredMessage(Message):
    @property
    def optional_message(self) -> TestRequired: ...
    @property
    def repeated_message(self) -> RepeatedCompositeFieldContainer[TestRequired]: ...
    @property
    def required_message(self) -> TestRequired: ...
    def __init__(
        self,
        required_message: TestRequired,
        optional_message: Optional[TestRequired] = ...,
        repeated_message: Optional[Iterable[TestRequired]] = ...,
    ) -> None: ...

class TestForeignNested(Message):
    @property
    def foreign_nested(self) -> TestAllTypes.NestedMessage: ...
    def __init__(self, foreign_nested: Optional[TestAllTypes.NestedMessage] = ...) -> None: ...

class TestEmptyMessage(Message):
    def __init__(self,) -> None: ...

class TestEmptyMessageWithExtensions(Message):
    def __init__(self,) -> None: ...

class TestMultipleExtensionRanges(Message):
    def __init__(self,) -> None: ...

class TestReallyLargeTagNumber(Message):
    a: int
    bb: int
    def __init__(self, a: Optional[int] = ..., bb: Optional[int] = ...) -> None: ...

class TestRecursiveMessage(Message):
    i: int
    @property
    def a(self) -> TestRecursiveMessage: ...
    def __init__(self, a: Optional[TestRecursiveMessage] = ..., i: Optional[int] = ...) -> None: ...

class TestMutualRecursionA(Message):
    class SubMessage(Message):
        @property
        def b(self) -> TestMutualRecursionB: ...
        def __init__(self, b: Optional[TestMutualRecursionB] = ...) -> None: ...
    class SubGroup(Message):
        @property
        def sub_message(self) -> TestMutualRecursionA.SubMessage: ...
        @property
        def not_in_this_scc(self) -> TestAllTypes: ...
        def __init__(
            self, sub_message: Optional[TestMutualRecursionA.SubMessage] = ..., not_in_this_scc: Optional[TestAllTypes] = ...
        ) -> None: ...
    @property
    def bb(self) -> TestMutualRecursionB: ...
    @property
    def subgroup(self) -> TestMutualRecursionA.SubGroup: ...
    def __init__(
        self, bb: Optional[TestMutualRecursionB] = ..., subgroup: Optional[TestMutualRecursionA.SubGroup] = ...
    ) -> None: ...

class TestMutualRecursionB(Message):
    optional_int32: int
    @property
    def a(self) -> TestMutualRecursionA: ...
    def __init__(self, a: Optional[TestMutualRecursionA] = ..., optional_int32: Optional[int] = ...) -> None: ...

class TestIsInitialized(Message):
    class SubMessage(Message):
        class SubGroup(Message):
            i: int
            def __init__(self, i: int) -> None: ...
        @property
        def subgroup(self) -> TestIsInitialized.SubMessage.SubGroup: ...
        def __init__(self, subgroup: Optional[TestIsInitialized.SubMessage.SubGroup] = ...) -> None: ...
    @property
    def sub_message(self) -> TestIsInitialized.SubMessage: ...
    def __init__(self, sub_message: Optional[TestIsInitialized.SubMessage] = ...) -> None: ...

class TestDupFieldNumber(Message):
    class Foo(Message):
        a: int
        def __init__(self, a: Optional[int] = ...) -> None: ...
    class Bar(Message):
        a: int
        def __init__(self, a: Optional[int] = ...) -> None: ...
    a: int
    @property
    def foo(self) -> TestDupFieldNumber.Foo: ...
    @property
    def bar(self) -> TestDupFieldNumber.Bar: ...
    def __init__(
        self, a: Optional[int] = ..., foo: Optional[TestDupFieldNumber.Foo] = ..., bar: Optional[TestDupFieldNumber.Bar] = ...
    ) -> None: ...

class TestEagerMessage(Message):
    @property
    def sub_message(self) -> TestAllTypes: ...
    def __init__(self, sub_message: Optional[TestAllTypes] = ...) -> None: ...

class TestLazyMessage(Message):
    @property
    def sub_message(self) -> TestAllTypes: ...
    def __init__(self, sub_message: Optional[TestAllTypes] = ...) -> None: ...

class TestNestedMessageHasBits(Message):
    class NestedMessage(Message):
        nestedmessage_repeated_int32: RepeatedScalarFieldContainer[int]
        @property
        def nestedmessage_repeated_foreignmessage(self) -> RepeatedCompositeFieldContainer[ForeignMessage]: ...
        def __init__(
            self,
            nestedmessage_repeated_int32: Optional[Iterable[int]] = ...,
            nestedmessage_repeated_foreignmessage: Optional[Iterable[ForeignMessage]] = ...,
        ) -> None: ...
    @property
    def optional_nested_message(self) -> TestNestedMessageHasBits.NestedMessage: ...
    def __init__(self, optional_nested_message: Optional[TestNestedMessageHasBits.NestedMessage] = ...) -> None: ...

class TestCamelCaseFieldNames(Message):
    PrimitiveField: int
    StringField: Text
    EnumField: ForeignEnum
    StringPieceField: Text
    CordField: Text
    RepeatedPrimitiveField: RepeatedScalarFieldContainer[int]
    RepeatedStringField: RepeatedScalarFieldContainer[Text]
    RepeatedEnumField: RepeatedScalarFieldContainer[ForeignEnum]
    RepeatedStringPieceField: RepeatedScalarFieldContainer[Text]
    RepeatedCordField: RepeatedScalarFieldContainer[Text]
    @property
    def MessageField(self) -> ForeignMessage: ...
    @property
    def RepeatedMessageField(self) -> RepeatedCompositeFieldContainer[ForeignMessage]: ...
    def __init__(
        self,
        PrimitiveField: Optional[int] = ...,
        StringField: Optional[Text] = ...,
        EnumField: Optional[ForeignEnum] = ...,
        MessageField: Optional[ForeignMessage] = ...,
        StringPieceField: Optional[Text] = ...,
        CordField: Optional[Text] = ...,
        RepeatedPrimitiveField: Optional[Iterable[int]] = ...,
        RepeatedStringField: Optional[Iterable[Text]] = ...,
        RepeatedEnumField: Optional[Iterable[ForeignEnum]] = ...,
        RepeatedMessageField: Optional[Iterable[ForeignMessage]] = ...,
        RepeatedStringPieceField: Optional[Iterable[Text]] = ...,
        RepeatedCordField: Optional[Iterable[Text]] = ...,
    ) -> None: ...

class TestFieldOrderings(Message):
    class NestedMessage(Message):
        oo: int
        bb: int
        def __init__(self, oo: Optional[int] = ..., bb: Optional[int] = ...) -> None: ...
    my_string: Text
    my_int: int
    my_float: float
    @property
    def optional_nested_message(self) -> TestFieldOrderings.NestedMessage: ...
    def __init__(
        self,
        my_string: Optional[Text] = ...,
        my_int: Optional[int] = ...,
        my_float: Optional[float] = ...,
        optional_nested_message: Optional[TestFieldOrderings.NestedMessage] = ...,
    ) -> None: ...

class TestExtensionOrderings1(Message):
    my_string: Text
    def __init__(self, my_string: Optional[Text] = ...) -> None: ...

class TestExtensionOrderings2(Message):
    class TestExtensionOrderings3(Message):
        my_string: Text
        def __init__(self, my_string: Optional[Text] = ...) -> None: ...
    my_string: Text
    def __init__(self, my_string: Optional[Text] = ...) -> None: ...

class TestExtremeDefaultValues(Message):
    escaped_bytes: bytes
    large_uint32: int
    large_uint64: int
    small_int32: int
    small_int64: int
    really_small_int32: int
    really_small_int64: int
    utf8_string: Text
    zero_float: float
    one_float: float
    small_float: float
    negative_one_float: float
    negative_float: float
    large_float: float
    small_negative_float: float
    inf_double: float
    neg_inf_double: float
    nan_double: float
    inf_float: float
    neg_inf_float: float
    nan_float: float
    cpp_trigraph: Text
    string_with_zero: Text
    bytes_with_zero: bytes
    string_piece_with_zero: Text
    cord_with_zero: Text
    replacement_string: Text
    def __init__(
        self,
        escaped_bytes: Optional[bytes] = ...,
        large_uint32: Optional[int] = ...,
        large_uint64: Optional[int] = ...,
        small_int32: Optional[int] = ...,
        small_int64: Optional[int] = ...,
        really_small_int32: Optional[int] = ...,
        really_small_int64: Optional[int] = ...,
        utf8_string: Optional[Text] = ...,
        zero_float: Optional[float] = ...,
        one_float: Optional[float] = ...,
        small_float: Optional[float] = ...,
        negative_one_float: Optional[float] = ...,
        negative_float: Optional[float] = ...,
        large_float: Optional[float] = ...,
        small_negative_float: Optional[float] = ...,
        inf_double: Optional[float] = ...,
        neg_inf_double: Optional[float] = ...,
        nan_double: Optional[float] = ...,
        inf_float: Optional[float] = ...,
        neg_inf_float: Optional[float] = ...,
        nan_float: Optional[float] = ...,
        cpp_trigraph: Optional[Text] = ...,
        string_with_zero: Optional[Text] = ...,
        bytes_with_zero: Optional[bytes] = ...,
        string_piece_with_zero: Optional[Text] = ...,
        cord_with_zero: Optional[Text] = ...,
        replacement_string: Optional[Text] = ...,
    ) -> None: ...

class SparseEnumMessage(Message):
    sparse_enum: TestSparseEnum
    def __init__(self, sparse_enum: Optional[TestSparseEnum] = ...) -> None: ...

class OneString(Message):
    data: Text
    def __init__(self, data: Optional[Text] = ...) -> None: ...

class MoreString(Message):
    data: RepeatedScalarFieldContainer[Text]
    def __init__(self, data: Optional[Iterable[Text]] = ...) -> None: ...

class OneBytes(Message):
    data: bytes
    def __init__(self, data: Optional[bytes] = ...) -> None: ...

class MoreBytes(Message):
    data: RepeatedScalarFieldContainer[bytes]
    def __init__(self, data: Optional[Iterable[bytes]] = ...) -> None: ...

class Int32Message(Message):
    data: int
    def __init__(self, data: Optional[int] = ...) -> None: ...

class Uint32Message(Message):
    data: int
    def __init__(self, data: Optional[int] = ...) -> None: ...

class Int64Message(Message):
    data: int
    def __init__(self, data: Optional[int] = ...) -> None: ...

class Uint64Message(Message):
    data: int
    def __init__(self, data: Optional[int] = ...) -> None: ...

class BoolMessage(Message):
    data: bool
    def __init__(self, data: Optional[bool] = ...) -> None: ...

class TestOneof(Message):
    class FooGroup(Message):
        a: int
        b: Text
        def __init__(self, a: Optional[int] = ..., b: Optional[Text] = ...) -> None: ...
    foo_int: int
    foo_string: Text
    @property
    def foo_message(self) -> TestAllTypes: ...
    @property
    def foogroup(self) -> TestOneof.FooGroup: ...
    def __init__(
        self,
        foo_int: Optional[int] = ...,
        foo_string: Optional[Text] = ...,
        foo_message: Optional[TestAllTypes] = ...,
        foogroup: Optional[TestOneof.FooGroup] = ...,
    ) -> None: ...

class TestOneofBackwardsCompatible(Message):
    class FooGroup(Message):
        a: int
        b: Text
        def __init__(self, a: Optional[int] = ..., b: Optional[Text] = ...) -> None: ...
    foo_int: int
    foo_string: Text
    @property
    def foo_message(self) -> TestAllTypes: ...
    @property
    def foogroup(self) -> TestOneofBackwardsCompatible.FooGroup: ...
    def __init__(
        self,
        foo_int: Optional[int] = ...,
        foo_string: Optional[Text] = ...,
        foo_message: Optional[TestAllTypes] = ...,
        foogroup: Optional[TestOneofBackwardsCompatible.FooGroup] = ...,
    ) -> None: ...

class TestOneof2(Message):
    class NestedEnum(int):
        @classmethod
        def Name(cls, number: int) -> bytes: ...
        @classmethod
        def Value(cls, name: bytes) -> TestOneof2.NestedEnum: ...
        @classmethod
        def keys(cls) -> List[bytes]: ...
        @classmethod
        def values(cls) -> List[TestOneof2.NestedEnum]: ...
        @classmethod
        def items(cls) -> List[Tuple[bytes, TestOneof2.NestedEnum]]: ...
    FOO: TestOneof2.NestedEnum
    BAR: TestOneof2.NestedEnum
    BAZ: TestOneof2.NestedEnum
    class FooGroup(Message):
        a: int
        b: Text
        def __init__(self, a: Optional[int] = ..., b: Optional[Text] = ...) -> None: ...
    class NestedMessage(Message):
        qux_int: int
        corge_int: RepeatedScalarFieldContainer[int]
        def __init__(self, qux_int: Optional[int] = ..., corge_int: Optional[Iterable[int]] = ...) -> None: ...
    foo_int: int
    foo_string: Text
    foo_cord: Text
    foo_string_piece: Text
    foo_bytes: bytes
    foo_enum: TestOneof2.NestedEnum
    bar_int: int
    bar_string: Text
    bar_cord: Text
    bar_string_piece: Text
    bar_bytes: bytes
    bar_enum: TestOneof2.NestedEnum
    baz_int: int
    baz_string: Text
    @property
    def foo_message(self) -> TestOneof2.NestedMessage: ...
    @property
    def foogroup(self) -> TestOneof2.FooGroup: ...
    @property
    def foo_lazy_message(self) -> TestOneof2.NestedMessage: ...
    def __init__(
        self,
        foo_int: Optional[int] = ...,
        foo_string: Optional[Text] = ...,
        foo_cord: Optional[Text] = ...,
        foo_string_piece: Optional[Text] = ...,
        foo_bytes: Optional[bytes] = ...,
        foo_enum: Optional[TestOneof2.NestedEnum] = ...,
        foo_message: Optional[TestOneof2.NestedMessage] = ...,
        foogroup: Optional[TestOneof2.FooGroup] = ...,
        foo_lazy_message: Optional[TestOneof2.NestedMessage] = ...,
        bar_int: Optional[int] = ...,
        bar_string: Optional[Text] = ...,
        bar_cord: Optional[Text] = ...,
        bar_string_piece: Optional[Text] = ...,
        bar_bytes: Optional[bytes] = ...,
        bar_enum: Optional[TestOneof2.NestedEnum] = ...,
        baz_int: Optional[int] = ...,
        baz_string: Optional[Text] = ...,
    ) -> None: ...

class TestRequiredOneof(Message):
    class NestedMessage(Message):
        required_double: float
        def __init__(self, required_double: float) -> None: ...
    foo_int: int
    foo_string: Text
    @property
    def foo_message(self) -> TestRequiredOneof.NestedMessage: ...
    def __init__(
        self,
        foo_int: Optional[int] = ...,
        foo_string: Optional[Text] = ...,
        foo_message: Optional[TestRequiredOneof.NestedMessage] = ...,
    ) -> None: ...

class TestPackedTypes(Message):
    packed_int32: RepeatedScalarFieldContainer[int]
    packed_int64: RepeatedScalarFieldContainer[int]
    packed_uint32: RepeatedScalarFieldContainer[int]
    packed_uint64: RepeatedScalarFieldContainer[int]
    packed_sint32: RepeatedScalarFieldContainer[int]
    packed_sint64: RepeatedScalarFieldContainer[int]
    packed_fixed32: RepeatedScalarFieldContainer[int]
    packed_fixed64: RepeatedScalarFieldContainer[int]
    packed_sfixed32: RepeatedScalarFieldContainer[int]
    packed_sfixed64: RepeatedScalarFieldContainer[int]
    packed_float: RepeatedScalarFieldContainer[float]
    packed_double: RepeatedScalarFieldContainer[float]
    packed_bool: RepeatedScalarFieldContainer[bool]
    packed_enum: RepeatedScalarFieldContainer[ForeignEnum]
    def __init__(
        self,
        packed_int32: Optional[Iterable[int]] = ...,
        packed_int64: Optional[Iterable[int]] = ...,
        packed_uint32: Optional[Iterable[int]] = ...,
        packed_uint64: Optional[Iterable[int]] = ...,
        packed_sint32: Optional[Iterable[int]] = ...,
        packed_sint64: Optional[Iterable[int]] = ...,
        packed_fixed32: Optional[Iterable[int]] = ...,
        packed_fixed64: Optional[Iterable[int]] = ...,
        packed_sfixed32: Optional[Iterable[int]] = ...,
        packed_sfixed64: Optional[Iterable[int]] = ...,
        packed_float: Optional[Iterable[float]] = ...,
        packed_double: Optional[Iterable[float]] = ...,
        packed_bool: Optional[Iterable[bool]] = ...,
        packed_enum: Optional[Iterable[ForeignEnum]] = ...,
    ) -> None: ...

class TestUnpackedTypes(Message):
    unpacked_int32: RepeatedScalarFieldContainer[int]
    unpacked_int64: RepeatedScalarFieldContainer[int]
    unpacked_uint32: RepeatedScalarFieldContainer[int]
    unpacked_uint64: RepeatedScalarFieldContainer[int]
    unpacked_sint32: RepeatedScalarFieldContainer[int]
    unpacked_sint64: RepeatedScalarFieldContainer[int]
    unpacked_fixed32: RepeatedScalarFieldContainer[int]
    unpacked_fixed64: RepeatedScalarFieldContainer[int]
    unpacked_sfixed32: RepeatedScalarFieldContainer[int]
    unpacked_sfixed64: RepeatedScalarFieldContainer[int]
    unpacked_float: RepeatedScalarFieldContainer[float]
    unpacked_double: RepeatedScalarFieldContainer[float]
    unpacked_bool: RepeatedScalarFieldContainer[bool]
    unpacked_enum: RepeatedScalarFieldContainer[ForeignEnum]
    def __init__(
        self,
        unpacked_int32: Optional[Iterable[int]] = ...,
        unpacked_int64: Optional[Iterable[int]] = ...,
        unpacked_uint32: Optional[Iterable[int]] = ...,
        unpacked_uint64: Optional[Iterable[int]] = ...,
        unpacked_sint32: Optional[Iterable[int]] = ...,
        unpacked_sint64: Optional[Iterable[int]] = ...,
        unpacked_fixed32: Optional[Iterable[int]] = ...,
        unpacked_fixed64: Optional[Iterable[int]] = ...,
        unpacked_sfixed32: Optional[Iterable[int]] = ...,
        unpacked_sfixed64: Optional[Iterable[int]] = ...,
        unpacked_float: Optional[Iterable[float]] = ...,
        unpacked_double: Optional[Iterable[float]] = ...,
        unpacked_bool: Optional[Iterable[bool]] = ...,
        unpacked_enum: Optional[Iterable[ForeignEnum]] = ...,
    ) -> None: ...

class TestPackedExtensions(Message):
    def __init__(self,) -> None: ...

class TestUnpackedExtensions(Message):
    def __init__(self,) -> None: ...

class TestDynamicExtensions(Message):
    class DynamicEnumType(int):
        @classmethod
        def Name(cls, number: int) -> bytes: ...
        @classmethod
        def Value(cls, name: bytes) -> TestDynamicExtensions.DynamicEnumType: ...
        @classmethod
        def keys(cls) -> List[bytes]: ...
        @classmethod
        def values(cls) -> List[TestDynamicExtensions.DynamicEnumType]: ...
        @classmethod
        def items(cls) -> List[Tuple[bytes, TestDynamicExtensions.DynamicEnumType]]: ...
    DYNAMIC_FOO: TestDynamicExtensions.DynamicEnumType
    DYNAMIC_BAR: TestDynamicExtensions.DynamicEnumType
    DYNAMIC_BAZ: TestDynamicExtensions.DynamicEnumType
    class DynamicMessageType(Message):
        dynamic_field: int
        def __init__(self, dynamic_field: Optional[int] = ...) -> None: ...
    scalar_extension: int
    enum_extension: ForeignEnum
    dynamic_enum_extension: TestDynamicExtensions.DynamicEnumType
    repeated_extension: RepeatedScalarFieldContainer[Text]
    packed_extension: RepeatedScalarFieldContainer[int]
    @property
    def message_extension(self) -> ForeignMessage: ...
    @property
    def dynamic_message_extension(self) -> TestDynamicExtensions.DynamicMessageType: ...
    def __init__(
        self,
        scalar_extension: Optional[int] = ...,
        enum_extension: Optional[ForeignEnum] = ...,
        dynamic_enum_extension: Optional[TestDynamicExtensions.DynamicEnumType] = ...,
        message_extension: Optional[ForeignMessage] = ...,
        dynamic_message_extension: Optional[TestDynamicExtensions.DynamicMessageType] = ...,
        repeated_extension: Optional[Iterable[Text]] = ...,
        packed_extension: Optional[Iterable[int]] = ...,
    ) -> None: ...

class TestRepeatedScalarDifferentTagSizes(Message):
    repeated_fixed32: RepeatedScalarFieldContainer[int]
    repeated_int32: RepeatedScalarFieldContainer[int]
    repeated_fixed64: RepeatedScalarFieldContainer[int]
    repeated_int64: RepeatedScalarFieldContainer[int]
    repeated_float: RepeatedScalarFieldContainer[float]
    repeated_uint64: RepeatedScalarFieldContainer[int]
    def __init__(
        self,
        repeated_fixed32: Optional[Iterable[int]] = ...,
        repeated_int32: Optional[Iterable[int]] = ...,
        repeated_fixed64: Optional[Iterable[int]] = ...,
        repeated_int64: Optional[Iterable[int]] = ...,
        repeated_float: Optional[Iterable[float]] = ...,
        repeated_uint64: Optional[Iterable[int]] = ...,
    ) -> None: ...

class TestParsingMerge(Message):
    class RepeatedFieldsGenerator(Message):
        class Group1(Message):
            @property
            def field1(self) -> TestAllTypes: ...
            def __init__(self, field1: Optional[TestAllTypes] = ...) -> None: ...
        class Group2(Message):
            @property
            def field1(self) -> TestAllTypes: ...
            def __init__(self, field1: Optional[TestAllTypes] = ...) -> None: ...
        @property
        def field1(self) -> RepeatedCompositeFieldContainer[TestAllTypes]: ...
        @property
        def field2(self) -> RepeatedCompositeFieldContainer[TestAllTypes]: ...
        @property
        def field3(self) -> RepeatedCompositeFieldContainer[TestAllTypes]: ...
        @property
        def group1(self) -> RepeatedCompositeFieldContainer[TestParsingMerge.RepeatedFieldsGenerator.Group1]: ...
        @property
        def group2(self) -> RepeatedCompositeFieldContainer[TestParsingMerge.RepeatedFieldsGenerator.Group2]: ...
        @property
        def ext1(self) -> RepeatedCompositeFieldContainer[TestAllTypes]: ...
        @property
        def ext2(self) -> RepeatedCompositeFieldContainer[TestAllTypes]: ...
        def __init__(
            self,
            field1: Optional[Iterable[TestAllTypes]] = ...,
            field2: Optional[Iterable[TestAllTypes]] = ...,
            field3: Optional[Iterable[TestAllTypes]] = ...,
            group1: Optional[Iterable[TestParsingMerge.RepeatedFieldsGenerator.Group1]] = ...,
            group2: Optional[Iterable[TestParsingMerge.RepeatedFieldsGenerator.Group2]] = ...,
            ext1: Optional[Iterable[TestAllTypes]] = ...,
            ext2: Optional[Iterable[TestAllTypes]] = ...,
        ) -> None: ...
    class OptionalGroup(Message):
        @property
        def optional_group_all_types(self) -> TestAllTypes: ...
        def __init__(self, optional_group_all_types: Optional[TestAllTypes] = ...) -> None: ...
    class RepeatedGroup(Message):
        @property
        def repeated_group_all_types(self) -> TestAllTypes: ...
        def __init__(self, repeated_group_all_types: Optional[TestAllTypes] = ...) -> None: ...
    @property
    def required_all_types(self) -> TestAllTypes: ...
    @property
    def optional_all_types(self) -> TestAllTypes: ...
    @property
    def repeated_all_types(self) -> RepeatedCompositeFieldContainer[TestAllTypes]: ...
    @property
    def optionalgroup(self) -> TestParsingMerge.OptionalGroup: ...
    @property
    def repeatedgroup(self) -> RepeatedCompositeFieldContainer[TestParsingMerge.RepeatedGroup]: ...
    def __init__(
        self,
        required_all_types: TestAllTypes,
        optional_all_types: Optional[TestAllTypes] = ...,
        repeated_all_types: Optional[Iterable[TestAllTypes]] = ...,
        optionalgroup: Optional[TestParsingMerge.OptionalGroup] = ...,
        repeatedgroup: Optional[Iterable[TestParsingMerge.RepeatedGroup]] = ...,
    ) -> None: ...

class TestCommentInjectionMessage(Message):
    a: Text
    def __init__(self, a: Optional[Text] = ...) -> None: ...

class FooRequest(Message):
    def __init__(self,) -> None: ...

class FooResponse(Message):
    def __init__(self,) -> None: ...

class FooClientMessage(Message):
    def __init__(self,) -> None: ...

class FooServerMessage(Message):
    def __init__(self,) -> None: ...

class BarRequest(Message):
    def __init__(self,) -> None: ...

class BarResponse(Message):
    def __init__(self,) -> None: ...

class TestJsonName(Message):
    field_name1: int
    fieldName2: int
    FieldName3: int
    _field_name4: int
    FIELD_NAME5: int
    field_name6: int
    def __init__(
        self,
        field_name1: Optional[int] = ...,
        fieldName2: Optional[int] = ...,
        FieldName3: Optional[int] = ...,
        _field_name4: Optional[int] = ...,
        FIELD_NAME5: Optional[int] = ...,
        field_name6: Optional[int] = ...,
    ) -> None: ...

class TestHugeFieldNumbers(Message):
    class OptionalGroup(Message):
        group_a: int
        def __init__(self, group_a: Optional[int] = ...) -> None: ...
    class StringStringMapEntry(Message):
        key: Text
        value: Text
        def __init__(self, key: Optional[Text] = ..., value: Optional[Text] = ...) -> None: ...
    optional_int32: int
    fixed_32: int
    repeated_int32: RepeatedScalarFieldContainer[int]
    packed_int32: RepeatedScalarFieldContainer[int]
    optional_enum: ForeignEnum
    optional_string: Text
    optional_bytes: bytes
    oneof_uint32: int
    oneof_string: Text
    oneof_bytes: bytes
    @property
    def optional_message(self) -> ForeignMessage: ...
    @property
    def optionalgroup(self) -> TestHugeFieldNumbers.OptionalGroup: ...
    @property
    def string_string_map(self) -> MutableMapping[Text, Text]: ...
    @property
    def oneof_test_all_types(self) -> TestAllTypes: ...
    def __init__(
        self,
        optional_int32: Optional[int] = ...,
        fixed_32: Optional[int] = ...,
        repeated_int32: Optional[Iterable[int]] = ...,
        packed_int32: Optional[Iterable[int]] = ...,
        optional_enum: Optional[ForeignEnum] = ...,
        optional_string: Optional[Text] = ...,
        optional_bytes: Optional[bytes] = ...,
        optional_message: Optional[ForeignMessage] = ...,
        optionalgroup: Optional[TestHugeFieldNumbers.OptionalGroup] = ...,
        string_string_map: Optional[Mapping[Text, Text]] = ...,
        oneof_uint32: Optional[int] = ...,
        oneof_test_all_types: Optional[TestAllTypes] = ...,
        oneof_string: Optional[Text] = ...,
        oneof_bytes: Optional[bytes] = ...,
    ) -> None: ...

class TestExtensionInsideTable(Message):
    field1: int
    field2: int
    field3: int
    field4: int
    field6: int
    field7: int
    field8: int
    field9: int
    field10: int
    def __init__(
        self,
        field1: Optional[int] = ...,
        field2: Optional[int] = ...,
        field3: Optional[int] = ...,
        field4: Optional[int] = ...,
        field6: Optional[int] = ...,
        field7: Optional[int] = ...,
        field8: Optional[int] = ...,
        field9: Optional[int] = ...,
        field10: Optional[int] = ...,
    ) -> None: ...



from typing import (
    Any,
    Callable,
    ContextManager,
    Dict,
    Generator,
    Iterable,
    List,
    Mapping,
    NoReturn,
    Optional,
    Sequence,
    Set,
    Tuple,
    TypeVar,
    Union,
)

from click.formatting import HelpFormatter
from click.parser import OptionParser

_CC = TypeVar("_CC", bound=Callable[[], Any])

def invoke_param_callback(
    callback: Callable[[Context, Parameter, Optional[str]], Any],
    ctx: Context,
    param: Parameter,
    value: Optional[str]
) -> Any:
    ...


def augment_usage_errors(
    ctx: Context, param: Optional[Parameter] = ...
) -> ContextManager[None]:
    ...


def iter_params_for_processing(
    invocation_order: Sequence[Parameter],
    declaration_order: Iterable[Parameter],
) -> Iterable[Parameter]:
    ...


class Context:
    parent: Optional[Context]
    command: Command
    info_name: Optional[str]
    params: Dict[Any, Any]
    args: List[str]
    protected_args: List[str]
    obj: Any
    default_map: Optional[Mapping[str, Any]]
    invoked_subcommand: Optional[str]
    terminal_width: Optional[int]
    max_content_width: Optional[int]
    allow_extra_args: bool
    allow_interspersed_args: bool
    ignore_unknown_options: bool
    help_option_names: List[str]
    token_normalize_func: Optional[Callable[[str], str]]
    resilient_parsing: bool
    auto_envvar_prefix: Optional[str]
    color: Optional[bool]
    _meta: Dict[str, Any]
    _close_callbacks: List[Any]
    _depth: int

    def __init__(
        self,
        command: Command,
        parent: Optional[Context] = ...,
        info_name: Optional[str] = ...,
        obj: Optional[Any] = ...,
        auto_envvar_prefix: Optional[str] = ...,
        default_map: Optional[Mapping[str, Any]] = ...,
        terminal_width: Optional[int] = ...,
        max_content_width: Optional[int] = ...,
        resilient_parsing: bool = ...,
        allow_extra_args: Optional[bool] = ...,
        allow_interspersed_args: Optional[bool] = ...,
        ignore_unknown_options: Optional[bool] = ...,
        help_option_names: Optional[List[str]] = ...,
        token_normalize_func: Optional[Callable[[str], str]] = ...,
        color: Optional[bool] = ...
    ) -> None:
        ...

    @property
    def meta(self) -> Dict[str, Any]:
        ...

    @property
    def command_path(self) -> str:
        ...

    def scope(self, cleanup: bool = ...) -> ContextManager[Context]:
        ...

    def make_formatter(self) -> HelpFormatter:
        ...

    def call_on_close(self, f: _CC) -> _CC: ...

    def close(self) -> None:
        ...

    def find_root(self) -> Context:
        ...

    def find_object(self, object_type: type) -> Any:
        ...

    def ensure_object(self, object_type: type) -> Any:
        ...

    def lookup_default(self, name: str) -> Any:
        ...

    def fail(self, message: str) -> NoReturn:
        ...

    def abort(self) -> NoReturn:
        ...

    def exit(self, code: Union[int, str] = ...) -> NoReturn:
        ...

    def get_usage(self) -> str:
        ...

    def get_help(self) -> str:
        ...

    def invoke(self, callback: Union[Command, Callable[..., Any]], *args, **kwargs) -> Any: ...
    def forward(self, callback: Union[Command, Callable[..., Any]], *args, **kwargs) -> Any: ...

class BaseCommand:
    allow_extra_args: bool
    allow_interspersed_args: bool
    ignore_unknown_options: bool
    name: str
    context_settings: Dict[Any, Any]
    def __init__(self, name: str, context_settings: Optional[Dict[Any, Any]] = ...) -> None: ...

    def get_usage(self, ctx: Context) -> str:
        ...

    def get_help(self, ctx: Context) -> str:
        ...

    def make_context(
        self, info_name: str, args: List[str], parent: Optional[Context] = ..., **extra
    ) -> Context:
        ...

    def parse_args(self, ctx: Context, args: List[str]) -> List[str]:
        ...

    def invoke(self, ctx: Context) -> Any:
        ...

    def main(
        self,
        args: Optional[List[str]] = ...,
        prog_name: Optional[str] = ...,
        complete_var: Optional[str] = ...,
        standalone_mode: bool = ...,
        **extra
    ) -> Any:
        ...

    def __call__(self, *args, **kwargs) -> Any:
        ...


class Command(BaseCommand):
    callback: Optional[Callable[..., Any]]
    params: List[Parameter]
    help: Optional[str]
    epilog: Optional[str]
    short_help: Optional[str]
    options_metavar: str
    add_help_option: bool
    hidden: bool
    deprecated: bool

    def __init__(
        self,
        name: str,
        context_settings: Optional[Dict[Any, Any]] = ...,
        callback: Optional[Callable[..., Any]] = ...,
        params: Optional[List[Parameter]] = ...,
        help: Optional[str] = ...,
        epilog: Optional[str] = ...,
        short_help: Optional[str] = ...,
        options_metavar: str = ...,
        add_help_option: bool = ...,
        hidden: bool = ...,
        deprecated: bool = ...,
    ) -> None:
        ...

    def get_params(self, ctx: Context) -> List[Parameter]:
        ...

    def format_usage(
        self,
        ctx: Context,
        formatter: HelpFormatter
    ) -> None:
        ...

    def collect_usage_pieces(self, ctx: Context) -> List[str]:
        ...

    def get_help_option_names(self, ctx: Context) -> Set[str]:
        ...

    def get_help_option(self, ctx: Context) -> Optional[Option]:
        ...

    def make_parser(self, ctx: Context) -> OptionParser:
        ...

    def get_short_help_str(self, limit: int = ...) -> str:
        ...

    def format_help(self, ctx: Context, formatter: HelpFormatter) -> None:
        ...

    def format_help_text(self, ctx: Context, formatter: HelpFormatter) -> None:
        ...

    def format_options(self, ctx: Context, formatter: HelpFormatter) -> None:
        ...

    def format_epilog(self, ctx: Context, formatter: HelpFormatter) -> None:
        ...


_T = TypeVar('_T')
_F = TypeVar('_F', bound=Callable[..., Any])


class MultiCommand(Command):
    no_args_is_help: bool
    invoke_without_command: bool
    subcommand_metavar: str
    chain: bool
    result_callback: Callable[..., Any]

    def __init__(
        self,
        name: Optional[str] = ...,
        invoke_without_command: bool = ...,
        no_args_is_help: Optional[bool] = ...,
        subcommand_metavar: Optional[str] = ...,
        chain: bool = ...,
        result_callback: Optional[Callable[..., Any]] = ...,
        **attrs
    ) -> None:
        ...

    def resultcallback(
        self, replace: bool = ...
    ) -> Callable[[_F], _F]:
        ...

    def format_commands(self, ctx: Context, formatter: HelpFormatter) -> None:
        ...

    def resolve_command(
        self, ctx: Context, args: List[str]
    ) -> Tuple[str, Command, List[str]]:
        ...

    def get_command(self, ctx: Context, cmd_name: str) -> Optional[Command]:
        ...

    def list_commands(self, ctx: Context) -> Iterable[str]:
        ...


class Group(MultiCommand):
    commands: Dict[str, Command]

    def __init__(
        self, name: Optional[str] = ..., commands: Optional[Dict[str, Command]] = ..., **attrs
    ) -> None:
        ...

    def add_command(self, cmd: Command, name: Optional[str] = ...):
        ...

    def command(self, *args, **kwargs) -> Callable[[Callable[..., Any]], Command]: ...
    def group(self, *args, **kwargs) -> Callable[[Callable[..., Any]], Group]: ...


class CommandCollection(MultiCommand):
    sources: List[MultiCommand]

    def __init__(
        self, name: Optional[str] = ..., sources: Optional[List[MultiCommand]] = ..., **attrs
    ) -> None:
        ...

    def add_source(self, multi_cmd: MultiCommand) -> None:
        ...


class _ParamType:
    name: str
    is_composite: bool
    envvar_list_splitter: Optional[str]

    def __call__(
        self,
        value: Optional[str],
        param: Optional[Parameter] = ...,
        ctx: Optional[Context] = ...,
    ) -> Any:
        ...

    def get_metavar(self, param: Parameter) -> str:
        ...

    def get_missing_message(self, param: Parameter) -> str:
        ...

    def convert(
        self,
        value: str,
        param: Optional[Parameter],
        ctx: Optional[Context],
    ) -> Any:
        ...

    def split_envvar_value(self, rv: str) -> List[str]:
        ...

    def fail(self, message: str, param: Optional[Parameter] = ..., ctx: Optional[Context] = ...) -> NoReturn:
        ...


# This type is here to resolve https://github.com/python/mypy/issues/5275
_ConvertibleType = Union[type, _ParamType, Tuple[Union[type, _ParamType], ...],
                         Callable[[str], Any], Callable[[Optional[str]], Any]]


class Parameter:
    param_type_name: str
    name: str
    opts: List[str]
    secondary_opts: List[str]
    type: _ParamType
    required: bool
    callback: Optional[Callable[[Context, Parameter, str], Any]]
    nargs: int
    multiple: bool
    expose_value: bool
    default: Any
    is_eager: bool
    metavar: Optional[str]
    envvar: Union[str, List[str], None]

    def __init__(
        self,
        param_decls: Optional[List[str]] = ...,
        type: Optional[_ConvertibleType] = ...,
        required: bool = ...,
        default: Optional[Any] = ...,
        callback: Optional[Callable[[Context, Parameter, str], Any]] = ...,
        nargs: Optional[int] = ...,
        metavar: Optional[str] = ...,
        expose_value: bool = ...,
        is_eager: bool = ...,
        envvar: Optional[Union[str, List[str]]] = ...
    ) -> None:
        ...

    @property
    def human_readable_name(self) -> str:
        ...

    def make_metavar(self) -> str:
        ...

    def get_default(self, ctx: Context) -> Any:
        ...

    def add_to_parser(self, parser: OptionParser, ctx: Context) -> None:
        ...

    def consume_value(self, ctx: Context, opts: Dict[str, Any]) -> Any:
        ...

    def type_cast_value(self, ctx: Context, value: Any) -> Any:
        ...

    def process_value(self, ctx: Context, value: Any) -> Any:
        ...

    def value_is_missing(self, value: Any) -> bool:
        ...

    def full_process_value(self, ctx: Context, value: Any) -> Any:
        ...

    def resolve_envvar_value(self, ctx: Context) -> str:
        ...

    def value_from_envvar(self, ctx: Context) -> Union[str, List[str]]:
        ...

    def handle_parse_result(
        self, ctx: Context, opts: Dict[str, Any], args: List[str]
    ) -> Tuple[Any, List[str]]:
        ...

    def get_help_record(self, ctx: Context) -> Tuple[str, str]:
        ...

    def get_usage_pieces(self, ctx: Context) -> List[str]:
        ...

    def get_error_hint(self, ctx: Context) -> str:
        ...


class Option(Parameter):
    prompt: str  # sic
    confirmation_prompt: bool
    hide_input: bool
    is_flag: bool
    flag_value: Any
    is_bool_flag: bool
    count: bool
    multiple: bool
    allow_from_autoenv: bool
    help: Optional[str]
    hidden: bool
    show_default: bool
    show_choices: bool
    show_envvar: bool

    def __init__(
        self,
        param_decls: Optional[List[str]] = ...,
        show_default: bool = ...,
        prompt: Union[bool, str] = ...,
        confirmation_prompt: bool = ...,
        hide_input: bool = ...,
        is_flag: Optional[bool] = ...,
        flag_value: Optional[Any] = ...,
        multiple: bool = ...,
        count: bool = ...,
        allow_from_autoenv: bool = ...,
        type: Optional[_ConvertibleType] = ...,
        help: Optional[str] = ...,
        hidden: bool = ...,
        show_choices: bool = ...,
        show_envvar: bool = ...,
        **attrs
    ) -> None:
        ...

    def prompt_for_value(self, ctx: Context) -> Any:
        ...


class Argument(Parameter):
    def __init__(
        self,
        param_decls: Optional[List[str]] = ...,
        required: Optional[bool] = ...,
        **attrs
    ) -> None:
        ...



"""
Searching for names with given scope and name. This is very central in Jedi and
Python. The name resolution is quite complicated with descripter,
``__getattribute__``, ``__getattr__``, ``global``, etc.

If you want to understand name resolution, please read the first few chapters
in http://blog.ionelmc.ro/2015/02/09/understanding-python-metaclasses/.

Flow checks
+++++++++++

Flow checks are not really mature. There's only a check for ``isinstance``.  It
would check whether a flow has the form of ``if isinstance(a, type_or_tuple)``.
Unfortunately every other thing is being ignored (e.g. a == '' would be easy to
check for -> a is a string). There's big potential in these checks.
"""

from parso.tree import search_ancestor
from parso.python.tree import Name

from jedi import settings
from jedi.inference.arguments import TreeArguments
from jedi.inference.value import iterable
from jedi.inference.base_value import NO_VALUES
from jedi.parser_utils import is_scope


def filter_name(filters, name_or_str):
    """
    Searches names that are defined in a scope (the different
    ``filters``), until a name fits.
    """
    string_name = name_or_str.value if isinstance(name_or_str, Name) else name_or_str
    names = []
    for filter in filters:
        names = filter.get(string_name)
        if names:
            break

    return list(_remove_del_stmt(names))


def _remove_del_stmt(names):
    # Catch del statements and remove them from results.
    for name in names:
        if name.tree_name is not None:
            definition = name.tree_name.get_definition()
            if definition is not None and definition.type == 'del_stmt':
                continue
        yield name


def check_flow_information(value, flow, search_name, pos):
    """ Try to find out the type of a variable just with the information that
    is given by the flows: e.g. It is also responsible for assert checks.::

        if isinstance(k, str):
            k.  # <- completion here

    ensures that `k` is a string.
    """
    if not settings.dynamic_flow_information:
        return None

    result = None
    if is_scope(flow):
        # Check for asserts.
        module_node = flow.get_root_node()
        try:
            names = module_node.get_used_names()[search_name.value]
        except KeyError:
            return None
        names = reversed([
            n for n in names
            if flow.start_pos <= n.start_pos < (pos or flow.end_pos)
        ])

        for name in names:
            ass = search_ancestor(name, 'assert_stmt')
            if ass is not None:
                result = _check_isinstance_type(value, ass.assertion, search_name)
                if result is not None:
                    return result

    if flow.type in ('if_stmt', 'while_stmt'):
        potential_ifs = [c for c in flow.children[1::4] if c != ':']
        for if_test in reversed(potential_ifs):
            if search_name.start_pos > if_test.end_pos:
                return _check_isinstance_type(value, if_test, search_name)
    return result


def _get_isinstance_trailer_arglist(node):
    if node.type in ('power', 'atom_expr') and len(node.children) == 2:
        # This might be removed if we analyze and, etc
        first, trailer = node.children
        if first.type == 'name' and first.value == 'isinstance' \
                and trailer.type == 'trailer' and trailer.children[0] == '(':
            return trailer
    return None


def _check_isinstance_type(value, node, search_name):
    lazy_cls = None
    trailer = _get_isinstance_trailer_arglist(node)
    if trailer is not None and len(trailer.children) == 3:
        arglist = trailer.children[1]
        args = TreeArguments(value.inference_state, value, arglist, trailer)
        param_list = list(args.unpack())
        # Disallow keyword arguments
        if len(param_list) == 2 and len(arglist.children) == 3:
            (key1, _), (key2, lazy_value_cls) = param_list
            if key1 is None and key2 is None:
                call = _get_call_string(search_name)
                is_instance_call = _get_call_string(arglist.children[0])
                # Do a simple get_code comparison of the strings . They should
                # just have the same code, and everything will be all right.
                # There are ways that this is not correct, if some stuff is
                # redefined in between. However here we don't care, because
                # it's a heuristic that works pretty well.
                if call == is_instance_call:
                    lazy_cls = lazy_value_cls
    if lazy_cls is None:
        return None

    value_set = NO_VALUES
    for cls_or_tup in lazy_cls.infer():
        if isinstance(cls_or_tup, iterable.Sequence) and cls_or_tup.array_type == 'tuple':
            for lazy_value in cls_or_tup.py__iter__():
                value_set |= lazy_value.infer().execute_with_values()
        else:
            value_set |= cls_or_tup.execute_with_values()
    return value_set


def _get_call_string(node):
    if node.parent.type == 'atom_expr':
        return _get_call_string(node.parent)

    code = ''
    leaf = node.get_first_leaf()
    end = node.get_last_leaf().end_pos
    while leaf.start_pos < end:
        code += leaf.value
        leaf = leaf.get_next_leaf()
    return code



from typing import Any, Iterable, Iterator, List, Optional, Union

from django.db.models.base import Model
from django.utils.functional import SimpleLazyObject
from django.utils.safestring import SafeText

def capfirst(x: Optional[str]) -> Optional[str]: ...

re_words: Any
re_chars: Any
re_tag: Any
re_newlines: Any
re_camel_case: Any

def wrap(text: str, width: int) -> str: ...

class Truncator(SimpleLazyObject):
    def __init__(self, text: Union[Model, str]) -> None: ...
    def add_truncation_text(self, text: str, truncate: Optional[str] = ...) -> str: ...
    def chars(self, num: int, truncate: Optional[str] = ..., html: bool = ...) -> str: ...
    def words(self, num: int, truncate: Optional[str] = ..., html: bool = ...) -> str: ...

def get_valid_filename(s: str) -> str: ...
def get_text_list(list_: List[str], last_word: str = ...) -> str: ...
def normalize_newlines(text: str) -> str: ...
def phone2numeric(phone: str) -> str: ...
def compress_string(s: bytes) -> bytes: ...

class StreamingBuffer:
    vals: List[bytes] = ...
    def __init__(self) -> None: ...
    def write(self, val: bytes) -> None: ...
    def read(self) -> bytes: ...
    def flush(self): ...
    def close(self): ...

def compress_sequence(sequence: Iterable[bytes]) -> Iterator[bytes]: ...

smart_split_re: Any

def smart_split(text: str) -> Iterator[str]: ...
def unescape_entities(text: str) -> str: ...
def unescape_string_literal(s: str) -> str: ...
def slugify(value: str, allow_unicode: bool = ...) -> SafeText: ...
def camel_case_to_spaces(value: str) -> str: ...

format_lazy: Any



# Stubs for zipfile

from typing import Callable, Dict, IO, Iterable, List, Optional, Text, Tuple, Type, Union, Sequence, Pattern
from types import TracebackType
import io
import os
import sys

if sys.version_info >= (3, 6):
    _Path = Union[os.PathLike[str], str]
else:
    _Path = Text
_SZI = Union[Text, ZipInfo]
_DT = Tuple[int, int, int, int, int, int]

if sys.version_info >= (3,):
    class BadZipFile(Exception): ...
    BadZipfile = BadZipFile
else:
    class BadZipfile(Exception): ...

error = BadZipfile

class LargeZipFile(Exception): ...

class ZipExtFile(io.BufferedIOBase):
    MAX_N: int = ...
    MIN_READ_SIZE: int = ...

    if sys.version_info < (3, 6):
        PATTERN: Pattern[str] = ...

    if sys.version_info >= (3, 7):
        MAX_SEEK_READ: int = ...

    newlines: Optional[List[bytes]]
    mode: str
    name: str
    def __init__(
        self,
        fileobj: IO[bytes],
        mode: str,
        zipinfo: ZipInfo,
        decrypter: Optional[Callable[[Sequence[int]], bytes]] = ...,
        close_fileobj: bool = ...,
    ) -> None: ...
    def __repr__(self) -> str: ...
    def peek(self, n: int = ...) -> bytes: ...
    def read1(self, n: Optional[int]) -> bytes: ...  # type: ignore

class ZipFile:
    filename: Optional[Text]
    debug: int
    comment: bytes
    filelist: List[ZipInfo]
    fp: IO[bytes]
    NameToInfo: Dict[Text, ZipInfo]
    start_dir: int  # undocumented
    if sys.version_info >= (3, 8):
        def __init__(
            self,
            file: Union[_Path, IO[bytes]],
            mode: Text = ...,
            compression: int = ...,
            allowZip64: bool = ...,
            compresslevel: Optional[int] = ...,
            *,
            strict_timestamps: bool = ...,
        ) -> None: ...
    elif sys.version_info >= (3, 7):
        def __init__(
            self,
            file: Union[_Path, IO[bytes]],
            mode: Text = ...,
            compression: int = ...,
            allowZip64: bool = ...,
            compresslevel: Optional[int] = ...,
        ) -> None: ...
    else:
        def __init__(
            self, file: Union[_Path, IO[bytes]], mode: Text = ..., compression: int = ..., allowZip64: bool = ...
        ) -> None: ...
    def __enter__(self) -> ZipFile: ...
    def __exit__(
        self, exc_type: Optional[Type[BaseException]], exc_val: Optional[BaseException], exc_tb: Optional[TracebackType]
    ) -> None: ...
    def close(self) -> None: ...
    def getinfo(self, name: Text) -> ZipInfo: ...
    def infolist(self) -> List[ZipInfo]: ...
    def namelist(self) -> List[Text]: ...
    def open(self, name: _SZI, mode: Text = ..., pwd: Optional[bytes] = ...) -> IO[bytes]: ...
    def extract(self, member: _SZI, path: Optional[_SZI] = ..., pwd: bytes = ...) -> str: ...
    def extractall(
        self, path: Optional[_Path] = ..., members: Optional[Iterable[Text]] = ..., pwd: Optional[bytes] = ...
    ) -> None: ...
    def printdir(self) -> None: ...
    def setpassword(self, pwd: bytes) -> None: ...
    def read(self, name: _SZI, pwd: Optional[bytes] = ...) -> bytes: ...
    def testzip(self) -> Optional[str]: ...
    def write(self, filename: _Path, arcname: Optional[_Path] = ..., compress_type: Optional[int] = ...) -> None: ...
    if sys.version_info >= (3,):
        def writestr(self, zinfo_or_arcname: _SZI, data: Union[bytes, str], compress_type: Optional[int] = ...) -> None: ...
    else:
        def writestr(self, zinfo_or_arcname: _SZI, bytes: bytes, compress_type: Optional[int] = ...) -> None: ...

class PyZipFile(ZipFile):
    if sys.version_info >= (3,):
        def __init__(
            self, file: Union[str, IO[bytes]], mode: str = ..., compression: int = ..., allowZip64: bool = ..., opimize: int = ...
        ) -> None: ...
        def writepy(self, pathname: str, basename: str = ..., filterfunc: Optional[Callable[[str], bool]] = ...) -> None: ...
    else:
        def writepy(self, pathname: Text, basename: Text = ...) -> None: ...

class ZipInfo:
    filename: Text
    date_time: _DT
    compress_type: int
    comment: bytes
    extra: bytes
    create_system: int
    create_version: int
    extract_version: int
    reserved: int
    flag_bits: int
    volume: int
    internal_attr: int
    external_attr: int
    header_offset: int
    CRC: int
    compress_size: int
    file_size: int
    def __init__(self, filename: Optional[Text] = ..., date_time: Optional[_DT] = ...) -> None: ...
    if sys.version_info >= (3, 6):
        def is_dir(self) -> bool: ...
        @classmethod
        def from_file(cls, filename: _Path, arcname: Optional[_Path] = ...) -> ZipInfo: ...
    def FileHeader(self, zip64: Optional[bool] = ...) -> bytes: ...

def is_zipfile(filename: Union[_Path, IO[bytes]]) -> bool: ...

ZIP_STORED: int
ZIP_DEFLATED: int
if sys.version_info >= (3, 3):
    ZIP_BZIP2: int
    ZIP_LZMA: int



from typing import Tuple, Pattern, List, Dict, Union, Text

_DEFAULT_DELIMITER: str

def emojize(
    string: str,
    use_aliases: bool = ...,
    delimiters: Tuple[str, str] = ...
) -> str: ...

def demojize(
    string: str,
    delimiters: Tuple[str, str] = ...
) -> str: ...

def get_emoji_regexp() -> Pattern[Text]: ...

def emoji_lis(string: str) -> List[Dict[str, Union[int, str]]]: ...



import concurrent.futures
import sys
from typing import (
    Any, TypeVar, Set, List, TextIO, Union, Tuple, Generic, Generator, Iterable, Awaitable, overload, Iterator, Optional,
)
from types import FrameType
from .events import AbstractEventLoop
from .futures import Future

if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal

_T = TypeVar('_T')
_T1 = TypeVar('_T1')
_T2 = TypeVar('_T2')
_T3 = TypeVar('_T3')
_T4 = TypeVar('_T4')
_T5 = TypeVar('_T5')
_FutureT = Union[Future[_T], Generator[Any, None, _T], Awaitable[_T]]

FIRST_EXCEPTION: str
FIRST_COMPLETED: str
ALL_COMPLETED: str

def as_completed(fs: Iterable[_FutureT[_T]], *, loop: Optional[AbstractEventLoop] = ...,
                 timeout: Optional[float] = ...) -> Iterator[Future[_T]]: ...
def ensure_future(coro_or_future: _FutureT[_T],
                  *, loop: Optional[AbstractEventLoop] = ...) -> Future[_T]: ...
# Prior to Python 3.7 'async' was an alias for 'ensure_future'.
# It became a keyword in 3.7.

# `gather()` actually returns a list with length equal to the number
# of tasks passed; however, Tuple is used similar to the annotation for
# zip() because typing does not support variadic type variables.  See
# typing PR #1550 for discussion.
@overload
def gather(coro_or_future1: _FutureT[_T1],
           *, loop: Optional[AbstractEventLoop] = ..., return_exceptions: Literal[False] = ...) -> Future[Tuple[_T1]]: ...
@overload
def gather(coro_or_future1: _FutureT[_T1], coro_or_future2: _FutureT[_T2],
           *, loop: Optional[AbstractEventLoop] = ..., return_exceptions: Literal[False] = ...) -> Future[Tuple[_T1, _T2]]: ...
@overload
def gather(coro_or_future1: _FutureT[_T1], coro_or_future2: _FutureT[_T2], coro_or_future3: _FutureT[_T3],
           *, loop: Optional[AbstractEventLoop] = ...,
           return_exceptions: Literal[False] = ...) -> Future[Tuple[_T1, _T2, _T3]]: ...
@overload
def gather(coro_or_future1: _FutureT[_T1], coro_or_future2: _FutureT[_T2], coro_or_future3: _FutureT[_T3],
           coro_or_future4: _FutureT[_T4],
           *, loop: Optional[AbstractEventLoop] = ...,
           return_exceptions: Literal[False] = ...) -> Future[Tuple[_T1, _T2, _T3, _T4]]: ...
@overload
def gather(coro_or_future1: _FutureT[_T1], coro_or_future2: _FutureT[_T2], coro_or_future3: _FutureT[_T3],
           coro_or_future4: _FutureT[_T4], coro_or_future5: _FutureT[_T5],
           *, loop: Optional[AbstractEventLoop] = ...,
           return_exceptions: Literal[False] = ...) -> Future[Tuple[_T1, _T2, _T3, _T4, _T5]]: ...
@overload
def gather(coro_or_future1: _FutureT[Any], coro_or_future2: _FutureT[Any], coro_or_future3: _FutureT[Any],
           coro_or_future4: _FutureT[Any], coro_or_future5: _FutureT[Any], coro_or_future6: _FutureT[Any],
           *coros_or_futures: _FutureT[Any],
           loop: Optional[AbstractEventLoop] = ..., return_exceptions: bool = ...) -> Future[List[Any]]: ...
@overload
def gather(coro_or_future1: _FutureT[_T1],
           *, loop: Optional[AbstractEventLoop] = ...,
           return_exceptions: bool = ...) -> Future[Tuple[Union[_T1, BaseException]]]: ...
@overload
def gather(coro_or_future1: _FutureT[_T1], coro_or_future2: _FutureT[_T2],
           *, loop: Optional[AbstractEventLoop] = ...,
           return_exceptions: bool = ...) -> Future[Tuple[Union[_T1, BaseException], Union[_T2, BaseException]]]: ...
@overload
def gather(coro_or_future1: _FutureT[_T1], coro_or_future2: _FutureT[_T2], coro_or_future3: _FutureT[_T3],
           *, loop: Optional[AbstractEventLoop] = ..., return_exceptions: bool = ...) -> Future[
    Tuple[Union[_T1, BaseException], Union[_T2, BaseException], Union[_T3, BaseException]]]: ...
@overload
def gather(coro_or_future1: _FutureT[_T1], coro_or_future2: _FutureT[_T2], coro_or_future3: _FutureT[_T3],
           coro_or_future4: _FutureT[_T4],
           *, loop: Optional[AbstractEventLoop] = ..., return_exceptions: bool = ...) -> Future[
    Tuple[Union[_T1, BaseException], Union[_T2, BaseException], Union[_T3, BaseException], Union[_T4, BaseException]]]: ...
@overload
def gather(coro_or_future1: _FutureT[_T1], coro_or_future2: _FutureT[_T2], coro_or_future3: _FutureT[_T3],
           coro_or_future4: _FutureT[_T4], coro_or_future5: _FutureT[_T5],
           *, loop: Optional[AbstractEventLoop] = ..., return_exceptions: bool = ...) -> Future[
    Tuple[Union[_T1, BaseException], Union[_T2, BaseException], Union[_T3, BaseException], Union[_T4, BaseException], Union[_T5, BaseException]]]: ...
def run_coroutine_threadsafe(coro: _FutureT[_T],
                             loop: AbstractEventLoop) -> concurrent.futures.Future[_T]: ...
def shield(arg: _FutureT[_T], *, loop: Optional[AbstractEventLoop] = ...) -> Future[_T]: ...
def sleep(delay: float, result: _T = ..., loop: Optional[AbstractEventLoop] = ...) -> Future[_T]: ...
def wait(fs: Iterable[_FutureT[_T]], *, loop: Optional[AbstractEventLoop] = ..., timeout: Optional[float] = ...,
         return_when: str = ...) -> Future[Tuple[Set[Future[_T]], Set[Future[_T]]]]: ...
def wait_for(fut: _FutureT[_T], timeout: Optional[float],
             *, loop: Optional[AbstractEventLoop] = ...) -> Future[_T]: ...

class Task(Future[_T], Generic[_T]):
    @classmethod
    def current_task(cls, loop: Optional[AbstractEventLoop] = ...) -> Task[Any]: ...
    @classmethod
    def all_tasks(cls, loop: Optional[AbstractEventLoop] = ...) -> Set[Task[Any]]: ...
    if sys.version_info >= (3, 8):
        def __init__(
            self,
            coro: Union[Generator[Any, None, _T], Awaitable[_T]],
            *,
            loop: AbstractEventLoop = ...,
            name: Optional[str] = ...,
        ) -> None: ...
    else:
        def __init__(self, coro: Union[Generator[Any, None, _T], Awaitable[_T]], *, loop: AbstractEventLoop = ...) -> None: ...
    def __repr__(self) -> str: ...
    if sys.version_info >= (3, 8):
        def get_coro(self) -> Any: ...
        def get_name(self) -> str: ...
        def set_name(self, value: object) -> None: ...
    def get_stack(self, *, limit: int = ...) -> List[FrameType]: ...
    def print_stack(self, *, limit: int = ..., file: TextIO = ...) -> None: ...
    def cancel(self) -> bool: ...
    def _step(self, value: Any = ..., exc: Exception = ...) -> None: ...
    def _wakeup(self, future: Future[Any]) -> None: ...

if sys.version_info >= (3, 7):
    def all_tasks(loop: Optional[AbstractEventLoop] = ...) -> Set[Task[Any]]: ...
    if sys.version_info >= (3, 8):
        def create_task(
            coro: Union[Generator[Any, None, _T], Awaitable[_T]], *, name: Optional[str] = ...,
        ) -> Task[Any]: ...
    else:
        def create_task(coro: Union[Generator[Any, None, _T], Awaitable[_T]]) -> Task[Any]: ...
    def current_task(loop: Optional[AbstractEventLoop] = ...) -> Optional[Task[Any]]: ...



# Stubs for gc

import sys
from typing import Any, Dict, List, Optional, Tuple


DEBUG_COLLECTABLE: int
DEBUG_LEAK: int
DEBUG_SAVEALL: int
DEBUG_STATS: int
DEBUG_UNCOLLECTABLE: int
callbacks: List[Any]
garbage: List[Any]

def collect(generations: int = ...) -> int: ...
def disable() -> None: ...
def enable() -> None: ...
def get_count() -> Tuple[int, int, int]: ...
def get_debug() -> int: ...
if sys.version_info >= (3, 8):
    def get_objects(generation: Optional[int] = ...) -> List[Any]: ...
else:
    def get_objects() -> List[Any]: ...
def get_referents(*objs: Any) -> List[Any]: ...
def get_referrers(*objs: Any) -> List[Any]: ...
def get_stats() -> List[Dict[str, Any]]: ...
def get_threshold() -> Tuple[int, int, int]: ...
def is_tracked(obj: Any) -> bool: ...
def isenabled() -> bool: ...
def set_debug(flags: int) -> None: ...
def set_threshold(threshold0: int, threshold1: int = ...,
                  threshold2: int = ...) -> None: ...
